#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/3 15:43
@File: pytest_image.py
@IDE: pycharm
@Description:
    测试图片模块
"""
import os.path

import numpy as np
import pytest
import torch
from PIL import Image as PILImage
from matplotlib import pyplot as plt
from nanoid import generate

from swanlab.data.modules import Image
from tutils import TEMP_PATH


def test_image_ok():
    # ---------------------------------- numpy输入 ----------------------------------

    np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8)
    mock = np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8)
    image = Image(data_or_path=mock)
    data, buffer = image.parse()
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more() is None

    # ---------------------------------- pil输入 ----------------------------------

    mock = PILImage.fromarray(np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8))
    image = Image(data_or_path=mock)
    data, buffer = image.parse()
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more() is None

    # ---------------------------------- 路径输入 ----------------------------------
    mock = PILImage.fromarray(np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8))
    path = os.path.join(TEMP_PATH, f"{generate()}.png")
    # 写入文件
    mock.save(path)
    audio = Image(data_or_path=path)
    data, buffer = audio.parse()
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None

    # ---------------------------------- pytorch tensor输入 ----------------------------------
    mock = torch.randn(4, 3, 256, 256)
    image = Image(data_or_path=mock)
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more() is None

    # ---------------------------------- plt输入 ----------------------------------
    x = [1, 2, 3]
    y = [2, 3, 5]
    plt.plot(x, y)
    image = Image(data_or_path=plt)
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more() is None


def test_image_caption():
    # ---------------------------------- numpy输入 ----------------------------------

    np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8)
    mock = np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8)
    image = Image(data_or_path=mock, caption="test")
    data, buffer = image.parse()
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more()["caption"] == "test"

    # ---------------------------------- pil输入 ----------------------------------

    mock = PILImage.fromarray(np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8))
    image = Image(data_or_path=mock, caption="test")
    data, buffer = image.parse()
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more()["caption"] == "test"

    # ---------------------------------- 路径输入 ----------------------------------
    mock = PILImage.fromarray(np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8))
    path = os.path.join(TEMP_PATH, f"{generate()}.png")
    # 写入文件
    mock.save(path)
    audio = Image(data_or_path=path, caption="test")
    data, buffer = audio.parse()
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more()["caption"] == "test"

    # ---------------------------------- pytorch tensor输入 ----------------------------------
    mock = torch.randn(4, 3, 256, 256)
    image = Image(data_or_path=mock, caption="test")
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more()["caption"] == "test"

    # ---------------------------------- plt输入 ----------------------------------
    x = [1, 2, 3]
    y = [2, 3, 5]
    plt.plot(x, y)
    image = Image(data_or_path=plt, caption="test")
    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(".png")
    assert buffer is not None
    assert image.get_more()["caption"] == "test"


def test_image_fail():
    # 错误的路径
    with pytest.raises(ValueError):
        Image(data_or_path="not_exist.png")
    # 不是图片
    path = os.path.join(TEMP_PATH, "test.png")
    with open(path, "w") as f:
        f.write("hello")
    with pytest.raises(ValueError):
        Image(data_or_path=path)

    # ---------------------------------- 矩阵错误 ----------------------------------

    # 通道数错误
    mock = np.random.randint(low=0, high=256, size=(100, 100, 2), dtype=np.uint8)
    with pytest.raises(TypeError):
        Image(data_or_path=mock)

    mock = np.random.randint(low=0, high=256, size=(100, 100, 5), dtype=np.uint8)
    with pytest.raises(TypeError):
        Image(data_or_path=mock)

    # 错误的矩阵形状
    mock = np.random.randint(low=0, high=256, size=(3, 100, 100), dtype=np.uint8)
    with pytest.raises(TypeError):
        Image(data_or_path=mock)

    mock = np.random.randint(low=0, high=256, size=(1, 100, 100, 3), dtype=np.uint8)
    with pytest.raises(TypeError):
        Image(data_or_path=mock)

    # ---------------------------------- 文件类型错误 ----------------------------------
    mock = np.random.randint(low=0, high=256, size=(100, 100, 2), dtype=np.uint8)
    with pytest.raises(ValueError):
        Image(mock, file_type="svg")

    # ---------------------------------- size错误 -------------------------------------
    mock = np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8)
    with pytest.raises(ValueError):
        Image(mock, size=(100, 100, 100))
    with pytest.raises(ValueError):
        Image(mock, size="hello")  # noqa


@pytest.mark.parametrize("file_type", Image.ACCEPT_FORMAT)
def test_image_file_type(file_type):
    """
    测试不同的file_type输入情况
    """
    mock = np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8)

    image = Image(mock, file_type=file_type)
    assert image.format == file_type

    data, buffer = image.parse()

    # 返回文件名称
    assert isinstance(data, str)
    assert data.endswith(f".{file_type}")
    assert buffer is not None
    assert image.get_more() is None


def test_image_size():
    """
    测试不同的size输入情况
    """
    mock = np.random.randint(low=0, high=256, size=(256, 512, 3), dtype=np.uint8)
    # 转为PIL图像后，size为(512, 256)

    image = Image(mock, size=None)
    assert image.image_size == (512, 256)

    image = Image(mock, size=(128, 128))
    assert image.image_size == (128, 128)

    image = Image(mock, size=(128, None))
    assert image.image_size == (128, 64)

    image = Image(mock, size=(None, 128))
    assert image.image_size == (256, 128)

    image = Image(mock, size=128)
    assert image.image_size == (128, 64)



"""
// @author: ComPleHN
// @file: graphic.vue
// @time: 2025/5/27 16:33
// @description: 本文件是对于echarts的 图形组件 图表测试
"""
# ---------------------------------------------- Graphic - Graphic_image_with_js ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid
from pyecharts.faker import Faker

bar = (
    Bar(init_opts=opts.InitOpts(chart_id="1234"))
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-Graphic Image（旋转功能）组件示例"),
        graphic_opts=[
            opts.GraphicImage(
                graphic_item=opts.GraphicItem(
                    id_="logo", right=20, top=20, z=-10, bounding="raw", origin=[75, 75]
                ),
                graphic_imagestyle_opts=opts.GraphicImageStyleOpts(
                    image="https://echarts.apache.org/zh/images/favicon.png",
                    width=150,
                    height=150,
                    opacity=0.4,
                ),
            )
        ],
    )
)
c1 = (
    Grid(init_opts=opts.InitOpts(chart_id="1234"))
    .add(
        chart=bar,
        grid_opts=opts.GridOpts(pos_left="5%", pos_right="4%", pos_bottom="5%"),
    )
    .add_js_funcs(
        """
        var rotation = 0;
        setInterval(function () {
            chart_1234.setOption({
                graphic: {
                    id: 'logo',
                    rotation: (rotation += Math.PI / 360) % (Math.PI * 2)
                }
            });
        }, 30);
    """
    )
)

# ---------------------------------------------- Graphic - Graphic_rect_bar1 ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.commons.utils import JsCode
from pyecharts.faker import Faker

c2 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-Graphic Rect+Text 2 组件示例"),
        graphic_opts=[
            opts.GraphicGroup(
                graphic_item=opts.GraphicItem(left="50%", top="15%"),
                children=[
                    opts.GraphicRect(
                        graphic_item=opts.GraphicItem(
                            z=100, left="center", top="middle"
                        ),
                        graphic_shape_opts=opts.GraphicShapeOpts(width=190, height=90),
                        graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                            fill="#fff",
                            stroke="#555",
                            line_width=2,
                            shadow_blur=8,
                            shadow_offset_x=3,
                            shadow_offset_y=3,
                            shadow_color="rgba(0,0,0,0.3)",
                        ),
                    ),
                    opts.GraphicText(
                        graphic_item=opts.GraphicItem(
                            left="center", top="middle", z=100
                        ),
                        graphic_textstyle_opts=opts.GraphicTextStyleOpts(
                            text=JsCode(
                                "['横轴表示数据类别',"
                                "'纵轴表示数值的值',"
                                "'这个文本块可以放在图中各',"
                                "'种位置'].join('\\n')"
                            ),
                            font="14px Microsoft YaHei",
                            graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                                fill="#333"
                            ),
                        ),
                    ),
                ],
            )
        ],
    )
)

# ---------------------------------------------- Graphic - Graphic_image ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c3 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-Graphic Image 组件示例"),
        graphic_opts=[
            opts.GraphicImage(
                graphic_item=opts.GraphicItem(
                    id_="logo", right=20, top=20, z=-10, bounding="raw", origin=[75, 75]
                ),
                graphic_imagestyle_opts=opts.GraphicImageStyleOpts(
                    image="https://echarts.apache.org/zh/images/favicon.png",
                    width=150,
                    height=150,
                    opacity=0.4,
                ),
            )
        ],
    )
)

# ---------------------------------------------- Graphic - Graphic_rect_bar0 ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.commons.utils import JsCode
from pyecharts.faker import Faker


c4 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-Graphic Rect+Text 1 组件示例"),
        graphic_opts=[
            opts.GraphicGroup(
                graphic_item=opts.GraphicItem(
                    rotation=JsCode("Math.PI / 4"),
                    bounding="raw",
                    right=110,
                    bottom=110,
                    z=100,
                ),
                children=[
                    opts.GraphicRect(
                        graphic_item=opts.GraphicItem(
                            left="center", top="center", z=100
                        ),
                        graphic_shape_opts=opts.GraphicShapeOpts(width=400, height=50),
                        graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                            fill="rgba(0,0,0,0.3)"
                        ),
                    ),
                    opts.GraphicText(
                        graphic_item=opts.GraphicItem(
                            left="center", top="center", z=100
                        ),
                        graphic_textstyle_opts=opts.GraphicTextStyleOpts(
                            text="pyecharts bar chart",
                            font="bold 26px Microsoft YaHei",
                            graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                                fill="#fff"
                            ),
                        ),
                    ),
                ],
            )
        ],
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="graphic",
    public=True,
)

swanlab.log(
    {
        "Graphic - Graphic_image_with_js": c1,
        "Graphic - Graphic_rect_bar1": c2,
        "Graphic - Graphic_image": c3,
        "Graphic - Graphic_rect_bar0": c4
    }
)



"""
MLflow synchronization module, used to synchronize MLflow operations to SwanLab.

Test script:
```python
import mlflow
import random
import swanlab

swanlab.sync_mlflow()

mlflow.set_experiment("mlflow_sync_test")

with mlflow.start_run(run_name="test_run"):
    mlflow.log_param("learning_rate", 0.01)
    mlflow.log_params({"batch_size": 32, "epochs": 10})
    
    for epoch in range(10):
        acc = 1 - 2 ** -epoch - random.random() / epoch
        loss = 2 ** -epoch + random.random() / epoch
        mlflow.log_metric("accuracy", acc, step=epoch)
        mlflow.log_metric("loss", loss, step=epoch)
        
        mlflow.log_metrics({
            "precision": acc * 0.9,
            "recall": acc * 0.8
        }, step=epoch)
"""

import swanlab
import os

def _extract_args(args, kwargs, param_names):
    """
    从args和kwargs中提取参数值的通用函数
    
    Args:
        args: 位置参数元组
        kwargs: 关键字参数字典
        param_names: 参数名称列表
    
    Returns:
        tuple: 按param_names顺序返回提取的参数值
    """
    values = []
    for i, name in enumerate(param_names):
        if len(args) > i:
            values.append(args[i])
        else:
            values.append(kwargs.get(name, None))
    return tuple(values)


def sync_mlflow(mode:str="cloud"):
    """
    Synchronize MLflow to SwanLab, supports recording parameters, metrics, and models
    
    Args:
        mode: "cloud", "local" 或 "disabled". https://docs.swanlab.cn/api/py-init.html
    
    Example:
    ```python
    import mlflow
    import random
    import swanlab
    
    swanlab.sync_mlflow()
    
    mlflow.set_experiment("my_experiment")
    
    with mlflow.start_run(run_name="my_run"):
        mlflow.log_param("learning_rate", 0.01)
        mlflow.log_params({"batch_size": 32, "epochs": 10})
        
        for epoch in range(10):
            acc = 1 - 2 ** -epoch - random.random() / epoch
            loss = 2 ** -epoch + random.random() / epoch
            mlflow.log_metric("accuracy", acc, step=epoch)
            mlflow.log_metric("loss", loss, step=epoch)
    ```
    """
    try:
        import mlflow
        from mlflow.tracking import MlflowClient
    except ImportError:
        raise ImportError("please install mlflow first, command: `pip install mlflow`")
    
    # 保存原始函数
    original_start_run = mlflow.start_run
    original_end_run = mlflow.end_run
    original_log_param = mlflow.log_param
    original_log_params = mlflow.log_params
    original_log_metric = mlflow.log_metric
    original_log_metrics = mlflow.log_metrics
    original_set_experiment = mlflow.set_experiment
    
    def patched_set_experiment(experiment_name, *args, **kwargs):
        """拦截设置实验名称的调用"""
        # 如果SwanLab尚未初始化，则将experiment_name设置为环境变量SWANLAB_MLFLOW_PROJECT
        if swanlab.data.get_run() is None:
            os.environ["SWANLAB_MLFLOW_PROJECT"] = experiment_name
        return original_set_experiment(experiment_name, *args, **kwargs)
    
    def patched_start_run(*args, **kwargs):
        """拦截开始运行的调用"""
        run_id, experiment_id, run_name, nested, tags = _extract_args(
            args, kwargs, ['run_id', 'experiment_id', 'run_name', 'nested', 'tags']
        )
        # 如果SwanLab尚未初始化，则初始化它
        if swanlab.data.get_run() is None:
            if os.environ.get("SWANLAB_MLFLOW_PROJECT"):
                swanlab.init(project=os.environ["SWANLAB_MLFLOW_PROJECT"], experiment_name=run_name, mode=mode)
            else:
                swanlab.init(experiment_name=run_name, mode=mode)
        elif run_name:  # 如果已初始化但提供了新的run_name
            # 更新配置以包含run_name
            swanlab.config.update({"mlflow_run_name": run_name})
        
        # 如果提供了tags，将其添加到SwanLab配置
        if tags:
            swanlab.config.update({"mlflow_tags": tags})
        
        return original_start_run(*args, **kwargs)
    
    def patched_end_run(*args, **kwargs):
        """拦截结束运行的调用"""
        status = _extract_args(args, kwargs, ['status'])[0]
        
        # 结束SwanLab运行
        swanlab.finish()
        
        return original_end_run(*args, **kwargs)
    
    def patched_log_param(key, value, *args, **kwargs):
        """拦截记录单个参数的调用"""
        # 将参数添加到SwanLab配置
        swanlab.config.update({key: value})
        
        return original_log_param(key, value, *args, **kwargs)
    
    def patched_log_params(params, *args, **kwargs):
        """拦截记录多个参数的调用"""
        # 将参数添加到SwanLab配置
        swanlab.config.update(params)
        
        return original_log_params(params, *args, **kwargs)
    
    def patched_log_metric(key, value, step=None, *args, **kwargs):
        """拦截记录单个指标的调用"""
        # 记录到SwanLab
        swanlab.log({key: value}, step=step)
        
        return original_log_metric(key, value, step=step, *args, **kwargs)
    
    def patched_log_metrics(metrics, step=None, *args, **kwargs):
        """拦截记录多个指标的调用"""
        # 记录到SwanLab
        swanlab.log(metrics, step=step)
        
        return original_log_metrics(metrics, step=step, *args, **kwargs)
    
    # 应用monkey patch
    mlflow.start_run = patched_start_run
    mlflow.end_run = patched_end_run
    mlflow.log_param = patched_log_param
    mlflow.log_params = patched_log_params
    mlflow.log_metric = patched_log_metric
    mlflow.log_metrics = patched_log_metrics
    mlflow.set_experiment = patched_set_experiment



#!/usr/bin/env python
r"""
@DATE: 2024/9/20 14:42
@File: test_namer.py
@IDE: pycharm
@Description:
    测试命名器、取色器
"""

import pytest

from swanlab.data import namer
from swanlab.data.namer import hex_to_rgb


def test_name_no_index():
    name = namer.generate_name()
    assert isinstance(name, str)


def test_name_with_index():
    name = namer.generate_name(4)
    assert isinstance(name, str)
    # 极大数
    name = namer.generate_name(999999999)
    assert isinstance(name, str)


def test_color_no_index():
    colors = namer.generate_colors()
    assert len(colors) == 2
    assert isinstance(colors, tuple)


def test_color_with_index():
    colors = namer.generate_colors(4)
    assert len(colors) == 2
    assert isinstance(colors, tuple)
    # 极大数
    colors = namer.generate_colors(999999999)
    assert len(colors) == 2
    assert isinstance(colors, tuple)


def test_hex_to_rgb_normal():
    """测试正常的hex颜色转换"""
    # 测试标准6位十六进制
    assert hex_to_rgb("#528d59") == (82, 141, 89)
    assert hex_to_rgb("528d59") == (82, 141, 89)

    # 测试3位简写形式
    assert hex_to_rgb("#fff") == (255, 255, 255)
    assert hex_to_rgb("fff") == (255, 255, 255)

    # 测试黑白色
    assert hex_to_rgb("#000000") == (0, 0, 0)
    assert hex_to_rgb("#FFFFFF") == (255, 255, 255)

    # 测试大小写混合
    assert hex_to_rgb("#fF0000") == (255, 0, 0)


def test_hex_to_rgb_edge_cases():
    """测试边界情况"""
    # 测试带空格的情况
    assert hex_to_rgb(" #528d59 ") == (82, 141, 89)
    assert hex_to_rgb(" 528d59 ") == (82, 141, 89)


def test_hex_to_rgb_invalid_input():
    """测试无效输入"""
    # 测试长度错误
    with pytest.raises(ValueError, match="Invalid hex color length"):
        hex_to_rgb("#12345")  # 5位

    with pytest.raises(ValueError, match="Invalid hex color length"):
        hex_to_rgb("1234567")  # 7位

    # 测试非法字符
    with pytest.raises(ValueError, match="Invalid hex color"):
        hex_to_rgb("#xyz123")

    with pytest.raises(ValueError, match="Invalid hex color"):
        hex_to_rgb("#12345g")

    # 测试空字符串
    with pytest.raises(ValueError, match="Invalid hex color length"):
        hex_to_rgb("")

    # 测试None
    with pytest.raises(AttributeError):
        hex_to_rgb(None)


def test_hex_to_rgb_rgb_bounds():
    """测试RGB值的范围"""
    # 测试最小值(0)
    result = hex_to_rgb("#000000")
    assert all(0 <= v <= 255 for v in result)

    # 测试最大值(255)
    result = hex_to_rgb("#FFFFFF")
    assert all(0 <= v <= 255 for v in result)

    # 测试中间值
    result = hex_to_rgb("#7F7F7F")
    assert all(0 <= v <= 255 for v in result)


def test_hex_to_rgb_basic_colors():
    """测试基本颜色"""
    assert hex_to_rgb("#FF0000") == (255, 0, 0)  # 红
    assert hex_to_rgb("#00FF00") == (0, 255, 0)  # 绿
    assert hex_to_rgb("#0000FF") == (0, 0, 255)  # 蓝
    assert hex_to_rgb("#FFFF00") == (255, 255, 0)  # 黄
    assert hex_to_rgb("#FF00FF") == (255, 0, 255)  # 品红
    assert hex_to_rgb("#00FFFF") == (0, 255, 255)  # 青


class TestGenerateRunId:

    def test_run_id_length_is_21_characters(self):
        run_id = namer.generate_run_id()
        assert len(run_id) == 21

    def test_run_id_contains_only_lowercase_and_digits(self):
        run_id = namer.generate_run_id()
        assert all(c.islower() or c.isdigit() for c in run_id)

    def test_run_id_is_random_each_time(self):
        run_id_1 = namer.generate_run_id()
        run_id_2 = namer.generate_run_id()
        assert run_id_1 != run_id_2



"""
@author: cunyue
@file: sync_tensorboardX.py
@time: 2025/1/20 17:15
@description: 测试同步tensorboardX
"""

import numpy as np
from tensorboardX import SummaryWriter

import swanlab

swanlab.sync_tensorboardX()
writer = SummaryWriter('runs/example')

writer.add_image('random_image', np.random.randint(0, 255, (3, 100, 100)), global_step=20)
writer.add_text('random_text', 'hello', global_step=10)

for i in range(100):
    scalar_value = np.random.rand()
    writer.add_scalar('random_scalar', scalar_value, i)
    writer.add_scalars('random_scalars', {'scalar1': scalar_value, 'scalar2': scalar_value * 2}, i)

writer.close()


from swanlab.core_python import get_client
from swanlab.data.store import get_run_store
from swanlab.env import get_mode
from swanlab.package import get_package_version


class SwanlabCloudConfig:
    """
    public data for the SwanLab project when running in cloud mode.
    """

    def __init__(self):
        self.__http = None
        if get_mode() == "cloud":
            try:
                self.__http = get_client()
            except ValueError:
                pass
        self.__available = self.__http is not None

    def __get_property_from_http(self, name: str):
        """
        Get the property from the http object.
        if the http object is None, it will be initialized.
        if initialization fails, it will return None.
        """
        if self.available:
            return getattr(self.__http, name)
        return None

    @property
    def available(self):
        """
        Whether the SwanLab is running in cloud mode.
        """
        return self.__available

    @property
    def project_name(self):
        """
        The name of the project. Equal to `run.public.project_name`.
        If swanlab is not running in cloud mode, it will return None.
        """
        return self.__get_property_from_http("projname")

    @property
    def project_url(self):
        """
        The url of the project. It is the url of the project page on the SwanLab.
        If swanlab is not running in cloud mode, it will return None.
        """
        if not self.available:
            return None
        return self.__get_property_from_http("web_proj_url")

    @property
    def experiment_name(self):
        """
        The name of the experiment. It may be different from the name of swanboard.
        """
        return self.__get_property_from_http("expname")

    @property
    def experiment_url(self):
        """
        The url of the experiment. It is the url of the experiment page on the SwanLab.
        """
        if not self.available:
            return None
        return self.__get_property_from_http("web_exp_url")


class SwanLabPublicConfig:
    """
    Public data for the SwanLab project.
    """

    def __init__(self):
        run_store = get_run_store()
        self.__run_store = run_store
        self.__project_name = run_store.project
        self.__cloud = SwanlabCloudConfig()

    def json(self):
        """
        Return a dict of the public config.
        This method is used to serialize the public config to json.
        """
        return {
            "project_name": self.project_name,
            "version": self.version,
            "run_id": self.run_id,
            "swanlog_dir": self.swanlog_dir,
            "run_dir": self.run_dir,
            "cloud": {
                "project_name": self.cloud.project_name,
                "project_url": self.cloud.project_url,
                "experiment_name": self.cloud.experiment_name,
                "experiment_url": self.cloud.experiment_url,
            },
        }

    @property
    def cloud(self):
        """
        The cloud configuration.
        """
        return self.__cloud

    @property
    def project_name(self):
        """
        The name of the project. Equal to `run.public.project_name`.
        """
        return self.__project_name

    @property
    def version(self) -> str:
        """
        The version of the SwanLab.
        """
        return get_package_version()

    @property
    def run_id(self) -> str:
        """
        The id of the run.
        """
        return self.__run_store.run_id

    @property
    def swanlog_dir(self) -> str:
        """
        The directory of the SwanLab log.
        """
        return self.__run_store.swanlog_dir

    @property
    def run_dir(self) -> str:
        """
        The directory of the run.
        """
        return self.__run_store.run_dir

    @property
    def backup_file(self) -> str:
        """
        The backup file of the run.
        """
        return self.__run_store.backup_file



"""
// @author: ComPleHN
// @file: surface3d.vue
// @time: 2025/5/29 10:41
// @description: 本文件是对于echarts的 3D曲面 图表的测试
"""
# ---------------------------------------------- Surface3d - Surface_wave ----------------------------------------------
import math
from typing import Union

import pyecharts.options as opts
from pyecharts.charts import Surface3D

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=surface-wave&gl=1

目前无法实现的功能:

1、暂时无法设置光滑表面 wireframe
2、暂时无法把 visualmap 进行隐藏
"""


def float_range(start: int, end: int, step: Union[int, float], round_number: int = 2):
    """
    浮点数 range
    :param start: 起始值
    :param end: 结束值
    :param step: 步长
    :param round_number: 精度
    :return: 返回一个 list
    """
    temp = []
    while True:
        if start < end:
            temp.append(round(start, round_number))
            start += step
        else:
            break
    return temp


def surface3d_data():
    for t0 in float_range(-3, 3, 0.05):
        y = t0
        for t1 in float_range(-3, 3, 0.05):
            x = t1
            z = math.sin(x**2 + y**2) * x / 3.14
            yield [x, y, z]


c1 = (
    Surface3D()
    .add(
        series_name="",
        shading="color",
        data=list(surface3d_data()),
        xaxis3d_opts=opts.Axis3DOpts(type_="value"),
        yaxis3d_opts=opts.Axis3DOpts(type_="value"),
        grid3d_opts=opts.Grid3DOpts(width=100, height=40, depth=100),
    )
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(
            dimension=2,
            max_=1,
            min_=-1,
            range_color=[
                "#313695",
                "#4575b4",
                "#74add1",
                "#abd9e9",
                "#e0f3f8",
                "#ffffbf",
                "#fee090",
                "#fdae61",
                "#f46d43",
                "#d73027",
                "#a50026",
            ],
        )
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="surface3d",
    public=True,
)

swanlab.log(
    {
        "Surface3d - Surface_wave": c1
    }
)



import numpy as np
import pytest

from swanlab.data.modules.object3d import Model3D, Object3D, PointCloud


class TestObject3D:
    def test_from_xyz_array(self, xyz_points):
        """测试从XYZ数组创建"""
        obj = Object3D(xyz_points)
        assert isinstance(obj, PointCloud)
        assert obj.points.shape == (4, 6)

    def test_from_xyzc_array(self, xyzc_points):
        """测试从XYZC数组创建"""
        obj = Object3D(xyzc_points)
        assert isinstance(obj, PointCloud)
        assert obj.points.shape == (4, 6)

    def test_from_xyzrgb_array(self, xyzrgb_points):
        """测试从XYZRGB数组创建"""
        obj = Object3D(xyzrgb_points)
        assert isinstance(obj, PointCloud)
        assert np.array_equal(obj.points, xyzrgb_points)

    def test_from_glb_file(self, test_files_dir):
        """测试从GLB文件创建"""
        glb_path = test_files_dir / "sample.glb"
        obj = Object3D(glb_path)
        assert isinstance(obj, Model3D)

    def test_from_swanlab_pts_json(self, test_files_dir):
        """测试从SwanLab点云文件创建"""
        pts_path = test_files_dir / "points.swanlab.pts.json"
        obj = Object3D(pts_path)
        assert isinstance(obj, PointCloud)

    def test_unsupported_format(self, test_files_dir):
        """测试不支持的格式"""
        with pytest.raises(ValueError):
            Object3D(np.zeros((4, 5)))  # 错误的点云形状

        # 测试不支持的文件类型
        wrong_file = test_files_dir / "wrong.txt"
        with pytest.raises(ValueError, match="Unsupported file type"):
            Object3D(wrong_file)

    def test_metadata_handling(self, xyz_points):
        """测试元数据处理"""
        obj = Object3D(xyz_points, caption="Test")
        assert obj.step is None
        assert obj.caption == "Test"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/5/12 14:20
@File: helper.py
@IDE: pycharm
@Description:
    回调函数操作员，批量处理回调函数的调用
"""
import threading
from enum import Enum
from typing import List, Union, Dict, Any, Tuple, Callable, Optional

from swanlab.data.run.webhook import try_send_webhook
from swanlab.log import swanlog
from swanlab.swanlab_settings import get_settings
from swanlab.toolkit import SwanKitCallback, MetricInfo, ColumnInfo, RuntimeInfo

OperatorReturnType = Dict[str, Any]


class SwanLabRunOperator(SwanKitCallback):
    """
    The SwanLabRunOperator is used to batch process callback instances, triggering them respectively in the order of
    injection when each occasion occurs. In design, we aim for isolation between each instance. However, if there are
    cases of shared scope, parameters, or variables between instances, developers should handle them appropriately,
    as SwanLabRunOperator itself will not address these situations for now.
    """

    def __init__(self, callbacks: Union[SwanKitCallback, List[SwanKitCallback]] = None):
        super(SwanLabRunOperator, self).__init__()
        callbacks = [callbacks] if isinstance(callbacks, SwanKitCallback) else callbacks
        self.callbacks = {}
        if callbacks is not None:
            for callback in callbacks:
                self.add_callback(callback)

    @property
    def disabled(self):
        """
        判断是否所有回调函数都已经被禁用
        FIXME 实际上这里与settings的一些属性有关，目前没问题，但是未来可能要改
        """
        return len(self.callbacks) == 0

    def add_callback(self, callback: SwanKitCallback):
        if not isinstance(callback, SwanKitCallback):
            raise TypeError(f"Unsupported callback type: {type(callback)}")
        if str(callback) == str(self) or callback in self.callbacks:
            raise ValueError(f"Cannot add the same callback instance: {callback}")
        self.callbacks[str(callback)] = callback

    def __str__(self):
        return "SwanLabRunOperator"

    def __run_all(self, method: str, *args, **kwargs):
        return {name: getattr(callback, method)(*args, **kwargs) for name, callback in self.callbacks.items()}

    @classmethod
    def parse_return(cls, ret: OperatorReturnType, key: str = None):
        """
        解析返回值，选择不为None的返回值，如果都为None，则返回None
        如果key不为None，则返回对应key的返回值
        :param ret: 返回值
        :param key: 返回值的key
        :return:
        """
        if key is not None:
            return ret[key]
        return next((v for v in ret.values() if v is not None), None)

    def on_init(self, proj_name: str, workspace: str, logdir: str = None, *args, **kwargs) -> OperatorReturnType:
        return self.__run_all(
            "on_init",
            proj_name,
            workspace,
            logdir=logdir,
            *args,
            **kwargs,
        )

    def before_run(self, *args, **kwargs):
        return self.__run_all("before_run", *args, **kwargs)

    def before_init_experiment(
        self,
        run_id: str,
        exp_name: str,
        description: str,
        colors: Tuple[str, str],
        *args,
        **kwargs,
    ):
        return self.__run_all(
            "before_init_experiment",
            run_id,
            exp_name,
            description,
            colors,
            *args,
            **kwargs,
        )

    def on_run(self, *args, **kwargs):
        self.__run_all("on_run", *args, **kwargs)
        try_send_webhook()

    def on_runtime_info_update(self, r: RuntimeInfo, *args, **kwargs):
        return self.__run_all("on_runtime_info_update", r, *args, **kwargs)

    def on_log(self, *args, **kwargs):
        return self.__run_all("on_log", *args, **kwargs)

    def on_metric_create(self, metric_info: MetricInfo, *args, **kwargs):
        return self.__run_all("on_metric_create", metric_info, *args, **kwargs)

    def on_column_create(self, column_info: ColumnInfo, *args, **kwargs):
        return self.__run_all("on_column_create", column_info, *args, **kwargs)

    def on_stop(self, error: str = None, epoch: int = None, *args, **kwargs):
        r = self.__run_all("on_stop", error=error, epoch=epoch, *args, **kwargs)
        # 清空所有注册的回调函数
        self.callbacks.clear()
        return r


class SwanLabRunState(Enum):
    """SwanLabRunState is an enumeration class that represents the state of the experiment.
    We Recommend that you use this enumeration class to represent the state of the experiment.
    """

    NOT_STARTED = -2
    SUCCESS = 1
    CRASHED = -1
    RUNNING = 0


class MonitorCron:
    """
    用于定时采集系统信息
    """

    def __init__(self, monitor_func: Callable):
        self.count = 0  # 计数器,执行次数
        self.monitor_interval = get_settings().hardware_interval  # 用户设置的采集间隔

        def _():
            monitor_func()
            self.count += 1
            self.timer = threading.Timer(self.sleep_time, _)
            self.timer.daemon = True
            self.timer.start()

        # 立即执行
        self.timer = threading.Timer(0, _)
        self.timer.daemon = True
        self.timer.start()

    def cancel(self):
        if self.timer is not None:
            self.timer.cancel()
            self.timer.join()

    @property
    def sleep_time(self):
        if self.monitor_interval is not None:
            return self.monitor_interval
        # 采集10次以下，每次间隔10秒
        # 采集10次到50次，每次间隔30秒
        # 采集50次以上，每次间隔60秒
        if self.count < 10:
            return 10
        elif self.count < 50:
            return 30
        else:
            return 60


def check_log_level(log_level: Optional[str]) -> str:
    """检查日志等级是否合法"""
    valid = ["debug", "info", "warning", "error", "critical"]
    if log_level is None:
        return "info"
    elif log_level.lower() in valid:
        return log_level.lower()
    else:
        swanlog.warning(f"The log level you provided is not valid, it has been set to {log_level}.")
        return "info"



"""
// @author: ComPleHN
// @file: funnel.vue
// @time: 2025/5/27 15:36
// @description: 本文件是对于echarts的 漏斗图 图表测试
"""
# ---------------------------------------------- Funnel - Funnel_sort_ascending ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Funnel
from pyecharts.faker import Faker

c1 = (
    Funnel()
    .add(
        "商品",
        [list(z) for z in zip(Faker.choose(), Faker.values())],
        sort_="ascending",
        label_opts=opts.LabelOpts(position="inside"),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Funnel-Sort（ascending）"))
)

# ---------------------------------------------- Funnel - Funnel_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Funnel
from pyecharts.faker import Faker

c2 = (
    Funnel()
    .add("商品", [list(z) for z in zip(Faker.choose(), Faker.values())])
    .set_global_opts(title_opts=opts.TitleOpts(title="Funnel-基本示例"))
)

# ---------------------------------------------- Funnel - Funnel_chart ----------------------------------------------
import pyecharts.options as opts
from pyecharts.charts import Funnel

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=funnel

目前无法实现的功能:

1、暂时无法对漏斗图的长宽等范围操作进行修改
"""
x_data = ["展现", "点击", "访问", "咨询", "订单"]
y_data = [100, 80, 60, 40, 20]

data = [[x_data[i], y_data[i]] for i in range(len(x_data))]

c3 = (
    Funnel()
    .add(
        series_name="",
        data_pair=data,
        gap=2,
        tooltip_opts=opts.TooltipOpts(trigger="item", formatter="{a} <br/>{b} : {c}%"),
        label_opts=opts.LabelOpts(is_show=True, position="inside"),
        itemstyle_opts=opts.ItemStyleOpts(border_color="#fff", border_width=1),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="漏斗图", subtitle="纯属虚构"))
)

# ---------------------------------------------- Funnel - Funnel_label_inside ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Funnel
from pyecharts.faker import Faker


c4 = (
    Funnel()
    .add(
        "商品",
        [list(z) for z in zip(Faker.choose(), Faker.values())],
        label_opts=opts.LabelOpts(position="inside"),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Funnel-Label（inside)"))
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="funnel",
    public=True,
)

swanlab.log(
    {
        "Funnel - Funnel_sort_ascending": c1,
        "Funnel - Funnel_base": c2,
        "Funnel - Funnel_chart": c3,
        "Funnel - Funnel_label_inside": c4
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/7 09:47
@File: test_experiment.py
@IDE: pycharm
@Description:
    测试开放API的实验相关接口
"""

import pytest

import tutils as T
from swanlab import OpenApi
from swanlab.api.types import ApiResponse, Experiment


@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_get_experiment():
    """
    获取一个实验的详细信息
    """
    api = OpenApi()
    exp_cuid = "test_cuid"
    res = api.get_experiment(project="test_project", exp_id=exp_cuid)
    assert isinstance(res, ApiResponse)
    if res.code == 200:
        assert isinstance(res.data, Experiment)

@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_list_experiments():
    """
    获取一个项目下的实验列表
    """
    api = OpenApi()
    res = api.list_experiments(project="SwanLab")
    assert isinstance(res, ApiResponse)
    if res.code == 200:
        assert isinstance(res.data, list)
        for item in res.data:
            assert isinstance(item, Experiment)

@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_get_summary():
    """
    获取一个实验的Summary信息
    """
    api = OpenApi()
    exp_cuid = "test_cuid"
    res = api.get_summary(project="test_project", exp_id=exp_cuid)
    assert isinstance(res, ApiResponse)
    if res.code == 200:
        assert isinstance(res.data, dict)

@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_delete_experiment():
    """
    删除一个实验
    """
    api = OpenApi()
    exp_cuid = "test_cuid"
    res = api.delete_experiment(project="test_project", exp_id=exp_cuid)
    assert isinstance(res, ApiResponse)
    assert res.code in [204, 404]



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AnyValue(_message.Message):
    __slots__ = ("string_value", "bool_value", "int_value", "double_value", "array_value", "kvlist_value", "bytes_value")
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    BOOL_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT_VALUE_FIELD_NUMBER: _ClassVar[int]
    DOUBLE_VALUE_FIELD_NUMBER: _ClassVar[int]
    ARRAY_VALUE_FIELD_NUMBER: _ClassVar[int]
    KVLIST_VALUE_FIELD_NUMBER: _ClassVar[int]
    BYTES_VALUE_FIELD_NUMBER: _ClassVar[int]
    string_value: str
    bool_value: bool
    int_value: int
    double_value: float
    array_value: ArrayValue
    kvlist_value: KeyValueList
    bytes_value: bytes
    def __init__(self, string_value: _Optional[str] = ..., bool_value: bool = ..., int_value: _Optional[int] = ..., double_value: _Optional[float] = ..., array_value: _Optional[_Union[ArrayValue, _Mapping]] = ..., kvlist_value: _Optional[_Union[KeyValueList, _Mapping]] = ..., bytes_value: _Optional[bytes] = ...) -> None: ...

class ArrayValue(_message.Message):
    __slots__ = ("values",)
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedCompositeFieldContainer[AnyValue]
    def __init__(self, values: _Optional[_Iterable[_Union[AnyValue, _Mapping]]] = ...) -> None: ...

class KeyValueList(_message.Message):
    __slots__ = ("values",)
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedCompositeFieldContainer[KeyValue]
    def __init__(self, values: _Optional[_Iterable[_Union[KeyValue, _Mapping]]] = ...) -> None: ...

class KeyValue(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: AnyValue
    def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[AnyValue, _Mapping]] = ...) -> None: ...



"""
Docs:https://docs.swanlab.cn/guide_cloud/integration/integration-pytorch-lightning.html
"""

import os
import importlib.util
from typing import Any, Dict, Optional, Union, Mapping, List
from argparse import Namespace
import packaging.version
from pathlib import Path

if importlib.util.find_spec("lightning"):
    import lightning.pytorch as pl

    from lightning.pytorch.loggers.logger import Logger, rank_zero_experiment

    from lightning.pytorch.utilities import rank_zero_only, rank_zero_warn
elif importlib.util.find_spec("pytorch_lightning"):
    import pytorch_lightning as pl

    if packaging.version.parse(pl.__version__) < packaging.version.parse("1.7"):
        from pytorch_lightning.loggers.base import (
            LightningLoggerBase as Logger,
            rank_zero_experiment,
        )
    else:
        from pytorch_lightning.loggers.logger import (
            Logger,
            rank_zero_experiment,
        )

    from pytorch_lightning.utilities import rank_zero_only, rank_zero_warn
else:
    raise RuntimeError(
        "This contrib module requires PyTorch Lightning to be installed. "
        "Please install it with command: \n pip install pytorch-lightning"
        "or \n pip install lightning"
    )
import swanlab
from ..data.run import SwanLabRun
from lightning_fabric.utilities.logger import _add_prefix, _convert_params, _sanitize_callable_params


"""
swanlab_logger = SwanLabLogger(
    project="example_project",
    description="example_description",
)
"""


class SwanLabLogger(Logger):
    def __init__(
        self,
        project: Optional[str] = None,
        workspace: Optional[str] = None,
        experiment_name: Optional[str] = None,
        description: Optional[str] = None,
        logdir: Optional[str] = None,
        mode: Optional[str] = None,
        save_dir: Union[str, Path] = ".",
        tags: Optional[List[str]] = None,
        **kwargs: Any,
    ):
        super().__init__()
        
        tags = tags or []
        tags.append("⚡️pytorch lightning") if "⚡️pytorch lightning" not in tags else None

        self._experiment = None

        self._swanlab_init: Dict[str, Any] = {
            "project": project,
            "workspace": workspace,
            "experiment_name": experiment_name,
            "description": description,
            "logdir": logdir,
            "mode": mode,
            "tags": tags,
        }

        self._swanlab_init.update(**kwargs)

        self._project = self._swanlab_init.get("project")
        self._workspace = self._swanlab_init.get("workspace")
        self._experiment_name = self._swanlab_init.get("experiment_name")
        self._description = self._swanlab_init.get("decsription")
        self._logdir = self._swanlab_init.get("logdir")
        self._mode = self._swanlab_init.get("mode")

        if save_dir is not None:
            save_dir = os.fspath(save_dir)
        self._save_dir = save_dir
    
    def update_config(self, config: Dict[str, Any]):
        swanlab.config.update(config)

    @property
    @rank_zero_experiment
    def experiment(self) -> SwanLabRun:
        """创建实验"""
        swanlab.config["FRAMEWORK"] = "⚡️pytorch_lightning"
        
        if swanlab.get_run() is None:
            self._experiment = swanlab.init(**self._swanlab_init)
        else:
            rank_zero_warn(
                "There is a swanlab experiment already in progress and newly created instances of `SwanLabLogger` will reuse"
                " this experiment. If this is not desired, call `swanlab.finish()` before instantiating `SwanLabLogger`."
            )
            self._experiment = swanlab.get_run()

        return self._experiment

    @rank_zero_only
    def log_hyperparams(self, params: Union[Dict[str, Any], Namespace]) -> None:
        """记录超参数(config)"""
        params = _convert_params(params)
        params = _sanitize_callable_params(params)

        self.experiment.config.update(params)

    @rank_zero_only
    def log_metrics(self, metrics: Mapping[str, float], step: Optional[int] = None) -> None:
        """记录标量指标"""

        assert rank_zero_only.rank == 0, "experiment tried to log from global_rank != 0"

        if step is not None:
            self.experiment.log(metrics, step=step)
        else:
            self.experiment.log(metrics)

    @rank_zero_only
    def log_image(self, key: str, images: List[Any], step: Optional[int] = None, **kwargs: Any) -> None:
        """Log images (tensors, numpy arrays, PIL Images or file paths).

        Optional kwargs are lists passed to each image (ex: caption).

        """
        if not isinstance(images, list):
            raise TypeError(f'Expected a list as "images", found {type(images)}')
        n = len(images)
        for k, v in kwargs.items():
            if len(v) != n:
                raise ValueError(f"Expected {n} items but only found {len(v)} for {k}")
        kwarg_list = [{k: kwargs[k][i] for k in kwargs} for i in range(n)]

        import swanlab

        metrics = {key: [swanlab.Image(img, **kwarg) for img, kwarg in zip(images, kwarg_list)]}
        self.log_metrics(metrics, step)  # type: ignore[arg-type]

    @rank_zero_only
    def log_audio(self, key: str, audios: List[Any], step: Optional[int] = None, **kwargs: Any) -> None:
        r"""Log audios (numpy arrays, or file paths).

        Args:
            key: The key to be used for logging the audio files
            audios: The list of audio file paths, or numpy arrays to be logged
            step: The step number to be used for logging the audio files
            \**kwargs: Optional kwargs are lists passed to each ``swanlab.Audio`` instance (ex: caption, sample_rate).

        Optional kwargs are lists passed to each audio (ex: caption, sample_rate).

        """
        if not isinstance(audios, list):
            raise TypeError(f'Expected a list as "audios", found {type(audios)}')
        n = len(audios)
        for k, v in kwargs.items():
            if len(v) != n:
                raise ValueError(f"Expected {n} items but only found {len(v)} for {k}")
        kwarg_list = [{k: kwargs[k][i] for k in kwargs} for i in range(n)]

        import swanlab

        metrics = {key: [swanlab.Audio(audio, **kwarg) for audio, kwarg in zip(audios, kwarg_list)]}
        self.log_metrics(metrics, step)  # type: ignore[arg-type]

    @rank_zero_only
    def log_text(self, key: str, texts: List[Any], step: Optional[int] = None, **kwargs: Any) -> None:
        r"""Log texts (numpy arrays, or file paths).

        Args:
            key: The key to be used for logging the string
            audios: The list of string to be logged
            step: The step number to be used for logging the string
            \**kwargs: Optional kwargs are lists passed to each ``swanlab.Audio`` instance (ex: caption, sample_rate).

        Optional kwargs are lists passed to each text (ex: caption).

        """
        if not isinstance(texts, list):
            raise TypeError(f'Expected a list as "texts", found {type(texts)}')
        n = len(texts)
        for k, v in kwargs.items():
            if len(v) != n:
                raise ValueError(f"Expected {n} items but only found {len(v)} for {k}")
        kwarg_list = [{k: kwargs[k][i] for k in kwargs} for i in range(n)]

        import swanlab

        metrics = {key: [swanlab.Text(text, **kwarg) for text, kwarg in zip(texts, kwarg_list)]}
        self.log_metrics(metrics, step)  # type: ignore[arg-type]

    @property
    def save_dir(self) -> Optional[str]:
        return self._save_dir

    @property
    def name(self) -> Optional[str]:
        return self._project

    @rank_zero_only
    def finalize(self, status: str) -> None:
        if status != "success":
            return

    @property
    def version(self) -> Optional[str]:
        # don't create an experiment if we don't have one
        return self.experiment.public.run_id if self._experiment is not None else None



"""
Docs: https://docs.swanlab.cn/guide_cloud/integration/integration-huggingface-transformers.html
"""

import os
from typing import Optional, List, Dict, Union, Any
import logging
import swanlab

try:
    from transformers.trainer_callback import TrainerCallback
    from transformers import modelcard
except ImportError:
    raise RuntimeError(
        "This contrib module requires Transformers to be installed. "
        "Please install it with command: \n pip install transformers"
    )


def rewrite_logs(d):
    new_d = {}
    eval_prefix = "eval_"
    eval_prefix_len = len(eval_prefix)
    test_prefix = "test_"
    test_prefix_len = len(test_prefix)
    for k, v in d.items():
        if k.startswith(eval_prefix):
            new_d["eval/" + k[eval_prefix_len:]] = v
        elif k.startswith(test_prefix):
            new_d["test/" + k[test_prefix_len:]] = v
        else:
            new_d["train/" + k] = v
    return new_d


class SwanLabCallback(TrainerCallback):
    def __init__(
        self,
        project: Optional[str] = None,
        workspace: Optional[str] = None,
        experiment_name: Optional[str] = None,
        description: Optional[str] = None,
        logdir: Optional[str] = None,
        mode: Optional[str] = None,
        tags: Optional[List[str]] = None,
        **kwargs: Any,
    ):
        """
        To use the `SwanLabCallback`, pass it into the `callback` parameter when initializing the `transformers.Trainer`.
        This allows the Trainer to utilize SwanLab's logging and monitoring functionalities during the training process.
        Parameters same with `swanlab.init`. Finds more informations 
        [here](https://docs.swanlab.cn/api/py-init.html#swanlab-init)

        Parameters
        ----------
        project : str, optional
            The project name of the current experiment, the default is None,
            which means the current project name is the same as the current working directory.
        workspace : str, optional
            Where the current project is located, it can be an organization or a user (currently only supports yourself).
            The default is None, which means the current entity is the same as the current user.
        experiment_name : str, optional
            The experiment name you currently have open. If this parameter is not provided,
            SwanLab will generate one for you by default.
        description : str, optional
            The experiment description you currently have open,
            used for a more detailed introduction or labeling of the current experiment.
            If you do not provide this parameter, you can modify it later in the web interface.
        logdir : str, optional
            The folder will store all the log information generated during the execution of SwanLab.
            If the parameter is None,
            SwanLab will generate a folder named "swanlog" in the same path as the code execution to store the data.
            If you want to visualize the generated log files,
            simply run the command `swanlab watch` in the same path where the code is executed
            (without entering the "swanlog" folder).
            You can also specify your own folder, but you must ensure that the folder exists and preferably does not contain
            anything other than data generated by Swanlab.
            In this case, if you want to view the logs,
            you must use something like `swanlab watch -l ./your_specified_folder` to specify the folder path.
        mode : str, optional
            Allowed values are 'cloud', 'cloud-only', 'local', 'disabled'.
            If the value is 'cloud', the data will be uploaded to the cloud and the local log will be saved.
            If the value is 'cloud-only', the data will only be uploaded to the cloud and the local log will not be saved.
            If the value is 'local', the data will only be saved locally and will not be uploaded to the cloud.
            If the value is 'disabled', the data will not be saved or uploaded, just parsing the data.
        """
        self._swanlab = swanlab
        self._initialized = False
        self._log_model = os.getenv("SWANLAB_LOG_MODEL", None)
        
        tags = tags or []
        tags.append("🤗transformers") if "🤗transformers" not in tags else None

        # for callback args
        self._swanlab_init: Dict[str, Any] = {
            "project": project,
            "workspace": workspace,
            "experiment_name": experiment_name,
            "description": description,
            "logdir": logdir,
            "mode": mode,
            "tags": tags,
        }
        self._swanlab_init.update(**kwargs)

    def setup(self, args, state, model, **kwargs):
        """
        Setup the optional SwanLab (*swanlab*) integration.

        One can subclass and override this method to customize the setup if needed. Find more information
        [here](https://docs.swanlab.cn/guide_cloud/integration/integration-huggingface-transformers.html).

        You can also override the following environment variables. Find more information about environment
        variables [here](https://docs.swanlab.cn/en/api/environment-variable.html#environment-variables)

        Environment:
        - **SWANLAB_API_KEY** (`str`, *optional*, defaults to `None`):
            Cloud API Key. During login, this environment variable is checked first. If it doesn't exist, the system
            checks if the user is already logged in. If not, the login process is initiated.

                - If a string is passed to the login interface, this environment variable is ignored.
                - If the user is already logged in, this environment variable takes precedence over locally stored
                login information.

        - **SWANLAB_PROJECT** (`str`, *optional*, defaults to `None`):
            Set this to a custom string to store results in a different project. If not specified, the name of the current
            running directory is used.

        - **SWANLAB_LOG_DIR** (`str`, *optional*, defaults to `swanlog`):
            This environment variable specifies the storage path for log files when running in local mode.
            By default, logs are saved in a folder named swanlog under the working directory.

        - **SWANLAB_MODE** (`Literal["local", "cloud", "disabled"]`, *optional*, defaults to `cloud`):
            SwanLab's parsing mode, which involves callbacks registered by the operator. Currently, there are three modes:
            local, cloud, and disabled. Note: Case-sensitive. Find more information
            [here](https://docs.swanlab.cn/en/api/py-init.html#swanlab-init)

        - **SWANLAB_LOG_MODEL** (`str`, *optional*, defaults to `None`):
            SwanLab does not currently support the save mode functionality.This feature will be available in a future
            release

        - **SWANLAB_WEB_HOST** (`str`, *optional*, defaults to `None`):
            Web address for the SwanLab cloud environment for private version (its free)

        - **SWANLAB_API_HOST** (`str`, *optional*, defaults to `None`):
            API address for the SwanLab cloud environment for private version (its free)

        """
        self._initialized = True

        if state.is_world_process_zero:
            logging.info('Automatic SwanLab logging enabled, to disable set os.environ["SWANLAB_MODE"] = "disabled"')
            combined_dict = {**args.to_dict()}

            if hasattr(model, "config") and model.config is not None:
                model_config = model.config if isinstance(model.config, dict) else model.config.to_dict()
                combined_dict = {**model_config, **combined_dict}
            if hasattr(model, "peft_config") and model.peft_config is not None:
                peft_config = model.peft_config
                combined_dict = {**{"peft_config": peft_config}, **combined_dict}
            trial_name = state.trial_name
            init_args = {}
            if trial_name is not None:
                init_args["experiment_name"] = f"{args.run_name}-{trial_name}"
            elif args.run_name is not None:
                init_args["experiment_name"] = args.run_name
            init_args["project"] = os.getenv("SWANLAB_PROJECT", None)

            if self._swanlab.get_run() is None:
                # ATTENTION: little differents in transformers
                init_args.update(self._swanlab_init)
                self._swanlab.init(
                    **init_args,
                )
            # show transformers logo!
            self._swanlab.config["FRAMEWORK"] = "🤗transformers"
            # add config parameters (run may have been created manually)
            self._swanlab.config.update(combined_dict)

            # add number of model parameters to swanlab config
            try:
                self._swanlab.config.update({"model_num_parameters": model.num_parameters()})
                # get peft model parameters
                if type(model).__name__ == "PeftModel" or type(model).__name__ == "PeftMixedModel":
                    trainable_params, all_param = model.get_nb_trainable_parameters()
                    self._swanlab.config.update({"peft_model_trainable_params": trainable_params})
                    self._swanlab.config.update({"peft_model_all_param": all_param})
            except AttributeError:
                logging.info("Could not log the number of model parameters in SwanLab due to an AttributeError.")

            # log the initial model architecture to an artifact
            if self._log_model is not None:
                logging.warning(
                    "SwanLab does not currently support the save mode functionality. "
                    "This feature will be available in a future release."
                )
                badge_markdown = (
                    f'[<img src="https://raw.githubusercontent.com/SwanHubX/assets/main/badge1.svg"'
                    f' alt="Visualize in SwanLab" height="28'
                    f'0" height="32"/>]({self._swanlab.get_run().public.cloud.exp_url})'
                )

                modelcard.AUTOGENERATED_TRAINER_COMMENT += f"\n{badge_markdown}"

    def update_config(self, config: Dict[str, Any]):
        """
        Update the SwanLab config.
        
        Example:
        ```python
        swanlab_callback = SwanLabCallback(...)
        swanlab_callback.update_config({"model_name": "qwen"})
        trainer = Trainer(..., callbacks=[swanlab_callback])
        ```
        """
        self._swanlab.config.update(config)

    def on_train_begin(self, args, state, control, model=None, **kwargs):
        if not self._initialized:
            self.setup(args, state, model, **kwargs)

    def on_train_end(self, args, state, control, model=None, processing_class=None, **kwargs):
        if self._log_model is not None and self._initialized and state.is_world_process_zero:
            logging.warning(
                "SwanLab does not currently support the save mode functionality. "
                "This feature will be available in a future release."
            )

    def on_log(self, args, state, control, model=None, logs=None, **kwargs):
        single_value_scalars = [
            "train_runtime",
            "train_samples_per_second",
            "train_steps_per_second",
            "train_loss",
            "total_flos",
        ]

        if not self._initialized:
            self.setup(args, state, model)
        if state.is_world_process_zero:
            for k, v in logs.items():
                if k in single_value_scalars:
                    self._swanlab.log({f"single_value/{k}": v}, step=state.global_step)
            non_scalar_logs = {k: v for k, v in logs.items() if k not in single_value_scalars}
            non_scalar_logs = rewrite_logs(non_scalar_logs)
            self._swanlab.log({**non_scalar_logs, "train/global_step": state.global_step}, step=state.global_step)

    def on_save(self, args, state, control, **kwargs):
        if self._log_model is not None and self._initialized and state.is_world_process_zero:
            logging.warning(
                "SwanLab does not currently support the save mode functionality. "
                "This feature will be available in a future release."
            )

    def on_predict(self, args, state, control, metrics, **kwargs):
        if not self._initialized:
            self.setup(args, state, **kwargs)
        if state.is_world_process_zero:
            metrics = rewrite_logs(metrics)
            self._swanlab.log(metrics)



"""
// @author: ComPleHN
// @file: graph.vue
// @time: 2025/5/27 16:04
// @description: 本文件是对于echarts的 关系图 图表测试,本文件有注意事项请看本目录下 README.md
"""
# ---------------------------------------------- Graph - Graph_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Graph

nodes = [
    {"name": "结点1", "symbolSize": 10},
    {"name": "结点2", "symbolSize": 20},
    {"name": "结点3", "symbolSize": 30},
    {"name": "结点4", "symbolSize": 40},
    {"name": "结点5", "symbolSize": 50},
    {"name": "结点6", "symbolSize": 40},
    {"name": "结点7", "symbolSize": 30},
    {"name": "结点8", "symbolSize": 20},
]
links = []
for i in nodes:
    for j in nodes:
        links.append({"source": i.get("name"), "target": j.get("name")})
c1 = (
    Graph()
    .add("", nodes, links, repulsion=8000)
    .set_global_opts(title_opts=opts.TitleOpts(title="Graph-基本示例"))
)

# ---------------------------------------------- Graph - Graph_with_edge_options ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Graph

nodes_data = [
    opts.GraphNode(name="结点1", symbol_size=10),
    opts.GraphNode(name="结点2", symbol_size=20),
    opts.GraphNode(name="结点3", symbol_size=30),
    opts.GraphNode(name="结点4", symbol_size=40),
    opts.GraphNode(name="结点5", symbol_size=50),
    opts.GraphNode(name="结点6", symbol_size=60),
]
links_data = [
    opts.GraphLink(source="结点1", target="结点2", value=2),
    opts.GraphLink(source="结点2", target="结点3", value=3),
    opts.GraphLink(source="结点3", target="结点4", value=4),
    opts.GraphLink(source="结点4", target="结点5", value=5),
    opts.GraphLink(source="结点5", target="结点6", value=6),
    opts.GraphLink(source="结点6", target="结点1", value=7),
]
c2 = (
    Graph()
    .add(
        "",
        nodes_data,
        links_data,
        repulsion=4000,
        edge_label=opts.LabelOpts(
            is_show=True, position="middle", formatter="{b} 的数据 {c}"
        ),
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Graph-GraphNode-GraphLink-WithEdgeLabel")
    )
)

# ---------------------------------------------- Graph - Graph_with_options ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Graph

nodes = [
    opts.GraphNode(name="结点1", symbol_size=10),
    opts.GraphNode(name="结点2", symbol_size=20),
    opts.GraphNode(name="结点3", symbol_size=30),
    opts.GraphNode(name="结点4", symbol_size=40),
    opts.GraphNode(name="结点5", symbol_size=50),
]
links = [
    opts.GraphLink(source="结点1", target="结点2"),
    opts.GraphLink(source="结点2", target="结点3"),
    opts.GraphLink(source="结点3", target="结点4"),
    opts.GraphLink(source="结点4", target="结点5"),
    opts.GraphLink(source="结点5", target="结点1"),
]
c3 = (
    Graph()
    .add("", nodes, links, repulsion=4000)
    .set_global_opts(title_opts=opts.TitleOpts(title="Graph-GraphNode-GraphLink"))
)

# ---------------------------------------------- Graph - Graph_weibo ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Graph

with open("../assets/echarts/graph/weibo.json", "r", encoding="utf-8") as f:
    j = json.load(f)
    nodes, links, categories, cont, mid, userl = j
c4 = (
    Graph()
    .add(
        "",
        nodes,
        links,
        categories,
        repulsion=50,
        linestyle_opts=opts.LineStyleOpts(curve=0.2),
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_global_opts(
        legend_opts=opts.LegendOpts(is_show=False),
        title_opts=opts.TitleOpts(title="Graph-微博转发关系图"),
    )
)

# ---------------------------------------------- Graph - Npm_dependencies ---------------------------------------------- 
import asyncio
from aiohttp import TCPConnector, ClientSession

import pyecharts.options as opts
from pyecharts.charts import Graph

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=graph-npm

目前无法实现的功能:

1、暂无
"""


async def get_json_data(url: str) -> dict:
    async with ClientSession(connector=TCPConnector(ssl=False)) as session:
        async with session.get(url=url) as response:
            return await response.json()


# 获取官方的数据
data = asyncio.run(
    get_json_data(
        url="https://echarts.apache.org/examples/data/asset/data/npmdepgraph.min10.json"
    )
)

nodes = [
    {
        "x": node["x"],
        "y": node["y"],
        "id": node["id"],
        "name": node["label"],
        "symbolSize": node["size"],
        "itemStyle": {"normal": {"color": node["color"]}},
    }
    for node in data["nodes"]
]

edges = [
    {"source": edge["sourceID"], "target": edge["targetID"]} for edge in data["edges"]
]


c5 = (
    Graph()
    .add(
        series_name="",
        nodes=nodes,
        links=edges,
        layout="none",
        is_roam=True,
        is_focusnode=True,
        label_opts=opts.LabelOpts(is_show=False),
        linestyle_opts=opts.LineStyleOpts(width=0.5, curve=0.3, opacity=0.7),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="NPM Dependencies"))
)

# ---------------------------------------------- Graph - Graph_les_miserables ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Graph


with open("../assets/echarts/graph/les-miserables.json", "r", encoding="utf-8") as f:
    j = json.load(f)
    nodes = j["nodes"]
    links = j["links"]
    categories = j["categories"]

c6 = (
    Graph(init_opts=opts.InitOpts(width="1000px", height="600px"))
    .add(
        "",
        nodes=nodes,
        links=links,
        categories=categories,
        layout="circular",
        is_rotate_label=True,
        linestyle_opts=opts.LineStyleOpts(color="source", curve=0.3),
        label_opts=opts.LabelOpts(position="right"),
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Graph-Les Miserables"),
        legend_opts=opts.LegendOpts(orient="vertical", pos_left="2%", pos_top="20%"),
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="graph",
    public=True,
)

swanlab.log(
    {
        "Graph - Graph_base": c1,
        "Graph - Graph_with_edge_options": c2,
        "Graph - Graph_with_options": c3,
        "Graph - Graph_weibo": c4,
        "Graph - Npm_dependencies": c5,
        "Graph - Graph_les_miserables": c6
    }
)




"""
@author: cunyue
@file: must.py
@time: 2025/7/3 14:11
@description: 测试 must 模式的 resume 功能
"""

import random

import swanlab

# 1. 初始化第一个实验
# 开启实验并记录
run = swanlab.init()
swanlab.log({"loss": 0.1, "accuracy": 0.9}, step=1)

# 2. 继续一个不存在的实验
try:
    run = swanlab.init(id="".join(random.choices("abcdefghijklmnopqrstuvwxyz0123456789", k=21)), resume='must')
except RuntimeError:
    pass
import time

time.sleep(5)
# 3. 继续第一个实验
run = swanlab.init(id=run.id, resume='must', reinit=True)
# 上传相同 step 的指标，此时报错
ll = run.log({"loss": 0.3, "accuracy": 0.7}, step=1)
assert ll["loss"].is_error, "Expected loss metric to be in error state due to duplicate step"
assert ll['accuracy'].is_error is not None, "Expected column error to be present for duplicate step"
ll = run.log({"loss": 0.5, "accuracy": 0.8})
assert not ll["loss"].is_error, "Expected loss metric to be logged successfully after reinit"
assert not ll['accuracy'].is_error, "Expected accuracy metric to be logged successfully after reinit"
assert ll['loss'].data == 0.5
assert ll['accuracy'].data == 0.8



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/1 17:36
@File: base.py
@IDE: pycharm
@Description:
    SwanLab OpenAPI API基类
"""
import json
from datetime import datetime, timezone
from typing import Any, Callable, List, Optional, Union

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from swanlab.api.types import ApiResponse
from swanlab.core_python import auth
from swanlab.log.log import SwanLog
from swanlab.package import get_package_version

_logger: Optional[SwanLog] = None

def get_logger(log_level: str = "info") -> SwanLog:
    global _logger
    if _logger is None:
        _logger = SwanLog("swanlab.openapi", log_level)
    else:
        _logger.level = log_level
    return _logger

def handle_response(resp: requests.Response) -> ApiResponse:
    try:
        data = resp.json() if resp.content else {}
    except (json.decoder.JSONDecodeError, requests.JSONDecodeError):
        return ApiResponse[str](
            code=resp.status_code,
            errmsg="sdk decode json error",
            data=resp.text
        )

    if not isinstance(data, (dict, list)):
        return ApiResponse[Any](
            code=resp.status_code,
            errmsg="sdk decode dict error",
            data=data
        )

    code = resp.status_code
    if 200 <= code < 300:
        message = ""
    else:
        message = f"api error: {resp.reason}. Trace id: {resp.headers.get('traceid')}"
    return ApiResponse(
        code=code,
        errmsg=message,
        data=data
    )


class ApiHTTP:
    REFRESH_TIME = 60 * 60 * 24 * 7  # 7天

    def __init__(self, login_info: auth.LoginInfo):
        self.__logger = get_logger()
        self.__login_info: auth.LoginInfo = login_info
        self.__session: requests.Session = self.__init_session()
        self.service: OpenApiService = OpenApiService(self)
    
    @property
    def session(self) -> requests.Session:
        """
        获取当前的requests.Session对象
        """
        return self.__session

    @property
    def username(self) -> str:
        """
        当前登录的用户名
        """
        return self.__login_info.username or ""

    @property
    def base_url(self):
        return self.__login_info.api_host

    @property
    def sid_expired_at(self):
        """
        获取sid的过期时间
        """
        return datetime.strptime(self.__login_info.expired_at or "", "%Y-%m-%dT%H:%M:%S.%fZ")

    def __init_session(self) -> requests.Session:
        session = requests.Session()
        session.mount(
            prefix="https://",
            adapter=HTTPAdapter(
                max_retries=Retry(
                    total=3,
                    backoff_factor=0.1,
                    status_forcelist=[500, 502, 503, 504],
                    allowed_methods=(["GET", "POST", "PUT", "DELETE", "PATCH"])
                )
            )
        )
        session.headers["swanlab-sdk"] = get_package_version()
        session.cookies.update({"sid": self.__login_info.sid or ""})
        return session

    def __before_request(self):
        if (self.sid_expired_at - datetime.now(timezone.utc).replace(tzinfo=None)).total_seconds() < self.REFRESH_TIME:
            self.__logger.debug("Refreshing sid...")
            self.__login_info = auth.login_by_key(self.__login_info.api_key or "", save=False)
            self.__session.headers["cookie"] = f"sid={self.__login_info.sid}"

    def get(self, url: str, params: dict) -> ApiResponse:
        self.__before_request()
        resp = self.__session.get(self.base_url + url, params=params)
        return handle_response(resp)

    def post(self, url: str, data: Union[dict, list], params: dict) -> ApiResponse:
        self.__before_request()
        resp = self.__session.post(self.base_url + url, json=data, params=params)
        return handle_response(resp)

    def delete(self, url: str, params: dict) -> ApiResponse:
        self.__before_request()
        resp = self.__session.delete(self.base_url + url, params=params)
        return handle_response(resp)

class OpenApiService:
    def __init__(self, http: ApiHTTP):
        self.http: ApiHTTP = http

    def get_exp_info(self, username: str, project: str, exp_id: str) -> ApiResponse[dict]:
        """
        获取实验信息
        """
        return self.http.get(f"/project/{username}/{project}/runs/{exp_id}", params={})

    def get_project_info(self, username: str, projname: str) -> ApiResponse[dict]:
        """
        获取项目详情
        """
        return self.http.get(f"/project/{username}/{projname}", params={})
    
    @staticmethod
    def fetch_paginated_api(
        api_func: Callable[..., ApiResponse],  # 分页 API 请求函数
        page_field: str = "page",
        size_field: str = "size",
        page_size: int = 10,
        *args, **kwargs
    ) -> ApiResponse[List]:
        """
        通用分页全量拉取函数

        Args:
            api_func (Callable): 分页 API 请求函数，应返回 ApiResponse[Pagination]
            page_field (str): 页码参数名，默认为 "page"
            size_field (str): 每页大小参数名，默认为 "size"
            page_size (int): 每页条数，默认为 10
            *args: 传递给 api_func 的位置参数
            **kwargs: 传递给 api_func 的关键字参数

        Returns:
            ApiResponse[list]: 返回所有分页数据组成的 ApiResponse
        """
        page = 1
        objs = []
        while True:
            kwargs.update({page_field: page, size_field: page_size})
            resp: ApiResponse = api_func(*args, **kwargs)
            if resp.errmsg:
                break

            objs.extend(resp.data.list)

            if len(objs) >= resp.data.total:
                break
            page += 1

        return ApiResponse[List](code=resp.code, errmsg=resp.errmsg, data=objs)

class ApiBase:
    def __init__(self, http: ApiHTTP):
        self.http: ApiHTTP = http



import pytest

from swanlab.data.run.metadata.hardware.dcu.hygon import DCUCollector, map_hygon_dcu

try:
    driver_version, dcu_map = map_hygon_dcu()
    max_mem_value = 0
    for dcu_id in dcu_map:
        mem_value = int(dcu_map[dcu_id]["memory"][:-2])
        max_mem_value = max(max_mem_value, mem_value)
    max_mem_value = max_mem_value * 1024
except Exception as e:
    driver_version = None
    dcu_map = None
    max_mem_value = None


@pytest.mark.skipif(driver_version is None, reason="Hygon DCU not found")
def test_hygon_dcu_info():
    collector = DCUCollector(dcu_map, max_mem_value)
    collector()
    assert collector.collect_num == 1



"""
Notification plugin for SwanLab.
Used for sending notifications to users.
"""

import base64
import hashlib
import hmac
import smtplib
from abc import ABC, abstractmethod
from datetime import datetime
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Optional, Dict, Any, Tuple

import requests

import swanlab
from swanlab.toolkit import SwanKitCallback


class PrintCallback(SwanKitCallback):
    """Basic callback for printing experiment status information."""

    def on_init(self, proj_name: str, workspace: str, logdir: str = None, *args, **kwargs):
        """Called when experiment initialization completes."""
        print(f"🚀 My callback: on_init: {proj_name}, {workspace}, {logdir}, {kwargs}")

    def on_stop(self, error: str = None, *args, **kwargs):
        """Called when experiment stops or encounters error."""
        status = f"with error: {error}" if error else "successfully"
        print(f"🚀 My callback: Experiment stopped {status}")

    def __str__(self):
        return "PrintCallback"


class EmailCallback(SwanKitCallback):
    """Email notification callback with bilingual support."""

    DEFAULT_TEMPLATES = {
        "en": {
            "subject_success": "SwanLab | Your experiment completed successfully",
            "subject_error": "SwanLab | Your experiment encountered an error",
            "body_success": "Your SwanLab experiment has completed successfully.\n",
            "body_error": "Your SwanLab experiment encountered an error: {error}\n",
            "link_text": "Project: {project}\nWorkspace: {workspace}\nName: {exp_name}\nDescription: {description}\nExperiment Link: {link}",
        },
        "zh": {
            "subject_success": "SwanLab | 您的实验已成功完成",
            "subject_error": "SwanLab | 您的实验遇到错误",
            "body_success": "您的 SwanLab 实验已成功完成。\n",
            "body_error": "您的 SwanLab 实验遇到错误: {error}\n",
            "link_text": "项目: {project}\n工作区: {workspace}\n实验名: {exp_name}\n描述: {description}\n实验链接: {link}",
        },
    }

    def __init__(
        self,
        sender_email: str,
        receiver_email: str,
        password: str,
        smtp_server: str = "smtp.gmail.com",
        port: int = 587,
        language: str = "en",
    ):
        """
        Initialize email callback configuration.

        :param sender_email: SMTP account email address
        :param receiver_email: Recipient email address
        :param password: SMTP account password
        :param smtp_server: SMTP server address
        :param port: SMTP server port
        :param language: Email content language (en/zh)
        """
        self.sender_email = sender_email
        self.receiver_email = receiver_email
        self.password = password
        self.smtp_server = smtp_server
        self.port = port
        self.language = language

    def _create_email_content(self, error: Optional[str] = None) -> Dict[str, str]:
        """Generate bilingual email content based on experiment status."""
        templates = self.DEFAULT_TEMPLATES[self.language]

        # Determine email subject and body based on error status
        if error:
            subject = templates["subject_error"]
            body = templates["body_error"].format(error=error)
        else:
            subject = templates["subject_success"]
            body = templates["body_success"]

        # Add experiment link if running in cloud mode
        exp_link = swanlab.get_url()
        if exp_link:
            body += templates["link_text"].format(
                project=self.project,
                workspace=self.workspace,
                exp_name=self.exp_name,
                description=self.description,
                link=exp_link,
            )

        return subject, body

    def send_email(self, subject: str, body: str) -> None:
        """Handle SMTP connection and email sending."""
        try:
            # Create email message
            msg = MIMEMultipart()
            msg["From"] = self.sender_email
            msg["To"] = self.receiver_email
            msg["Subject"] = subject
            msg.attach(MIMEText(body, "plain", "utf-8"))
            # Establish secure connection
            with smtplib.SMTP(self.smtp_server, self.port) as server:
                server.starttls()
                server.login(self.sender_email, self.password)
                server.send_message(msg)
                print("✅ Email sent successfully!")

        except smtplib.SMTPException as e:
            print(f"❌ Email sending failed: {str(e)}")

    def on_init(self, proj_name: str, workspace: str, logdir: str = None, *args, **kwargs):
        self.project = proj_name
        self.workspace = workspace

    def before_init_experiment(
        self,
        run_id: str,
        exp_name: str,
        description: str,
        colors: Tuple[str, str],
        *args,
        **kwargs,
    ):
        self.run_id = run_id
        self.exp_name = exp_name
        self.description = description

    def on_stop(self, error: str = None, *args, **kwargs):
        """Trigger email notification when experiment stops."""
        print("📧 Preparing email notification...")
        subject, body = self._create_email_content(error)
        self.send_email(subject, body)

    def __str__(self):
        return "EmailCallback"


WEBHOOK_DEFAULT_TEMPLATES = {
    "en": {
        "title": "SwanLab Message Notification\n",
        "msg_success": "SwanLab | Your experiment completed successfully\n",
        "msg_error": "Your SwanLab experiment encountered an error: {error}\n",
        "link_text": "Project: {project}\nWorkspace: {workspace}\nName: {exp_name}\nDescription: {description}\nExperiment Link: {link}",
    },
    "zh": {
        "title": "SwanLab 消息通知\n",
        "msg_success": "SwanLab | 您的实验已成功完成\n",
        "msg_error": "您的 SwanLab 实验遇到错误: {error}\n",
        "link_text": "项目: {project}\n工作区: {workspace}\n实验名: {exp_name}\n描述: {description}\n实验链接: {link}",
    },
}


class WebhookCallback(SwanKitCallback, ABC):
    """抽象基类，用于各种webhook通知回调"""

    def __init__(self, webhook_url: str, secret: Optional[str] = None, language: str = "zh"):
        self.webhook_url = webhook_url
        self.secret = secret
        self.language = language
        self._init_bot()

    @abstractmethod
    def _init_bot(self) -> None:
        """初始化机器人实例"""
        pass

    def _create_content(self, error: Optional[str] = None) -> str:
        """创建通知内容"""
        templates = WEBHOOK_DEFAULT_TEMPLATES[self.language]
        content_text: str = f"{templates['title']}"
        if error:
            content_text += templates["msg_error"].format(error=error)
        else:
            content_text += templates["msg_success"]
        exp_link = swanlab.get_url()
        if exp_link:
            content_text += templates["link_text"].format(
                project=self.project,
                workspace=self.workspace,
                exp_name=self.exp_name,
                description=self.description,
                link=exp_link,
            )
        return content_text

    @abstractmethod
    def send_msg(self, content: str) -> None:
        """发送消息的具体实现"""
        pass

    def on_init(self, proj_name: str, workspace: str, logdir: str = None, *args, **kwargs):
        self.project = proj_name
        self.workspace = workspace

    def before_init_experiment(
        self,
        run_id: str,
        exp_name: str,
        description: str,
        colors: Tuple[str, str],
        *args,
        **kwargs,
    ):
        self.run_id = run_id
        self.exp_name = exp_name
        self.description = description

    def on_stop(self, error: str = None, *args, **kwargs):
        print(f"🤖 Preparing {self.__class__.__name__} notification...")
        content = self._create_content(error)
        self.send_msg(content)


class LarkBot:
    def __init__(self, webhook_url: str, secret: Optional[str] = None):
        self.webhook_url = webhook_url
        self.secret = secret

    def gen_sign(self, timesteamp: int) -> str:
        """
        docs: https://open.feishu.cn/document/client-docs/bot-v3/add-custom-bot?lang=zh-CN#9ff32e8e
        If the user has configured the signature verification function, this method is required to generate the signature
        """
        if not self.secret:
            raise ValueError("secret is required")
        string_to_sign: str = f"{timesteamp}\n{self.secret}"
        hmac_code = hmac.new(string_to_sign.encode("utf-8"), digestmod=hashlib.sha256).digest()
        return base64.b64encode(hmac_code).decode("utf-8")


class LarkCallback(WebhookCallback):
    """Lark notification callback with bilingual support."""

    def _init_bot(self) -> None:
        self.bot = LarkBot(self.webhook_url, self.secret)

    def send_msg(self, content: str) -> None:
        timestamp: int = int(datetime.now().timestamp())
        json_params: Dict[str, Any] = {
            "timestamp": timestamp,
            "msg_type": "text",
            "content": {"text": content},
        }
        if self.bot.secret:
            json_params["sign"] = self.bot.gen_sign(timestamp)
        resp = requests.post(self.bot.webhook_url, json=json_params)
        resp.raise_for_status()
        result: Dict[str, Any] = resp.json()
        if result.get("code") and result["code"] != 0:
            print(f"❌ LarkBot sending failed: {result.get('msg')}")
            return
        print("✅ LarkBot sending successfully")

    def __str__(self):
        return "LarkBotCallback"


class DingTalkBot:
    """DingTalk notification callback with bilingual support."""

    def __init__(self, webhook_url: str, secret: Optional[str] = None):
        self.webhook_url = webhook_url
        self.secret = secret
        self.start_time = datetime.now().timestamp()  # 加签时，请求时间戳与请求时间不能超过1小时，用于定时更新签名
        if self.secret is not None and self.secret.startswith("SEC"):
            self.update_webhook_url()

    def _check_sign(self) -> None:
        """
        DingTalk 中的签名和时间戳需要放在 url 中，且签名中的时间戳与请求时间戳的差值不能超过 1 小时
        """
        ts_now = datetime.now().timestamp()
        if ts_now - self.start_time >= 3600 and self.secret is not None and self.secret.startswith("SEC"):
            self.start_time = ts_now
            self.update_webhook_url()

    def update_webhook_url(self) -> None:
        """
        DingTalk 中的 webhook url 需要更新签名和时间戳
        """
        """
        docs: https://open.dingtalk.com/document/
        DingTalk 中的签名和时间戳需要放在 url 中，且签名中的时间戳与请求时间戳的差值不能超过 1 小时
        """
        if (not self.secret) or (not self.secret.startswith("SEC")):
            raise ValueError("secret is invalid")
        timestamp = round(self.start_time * 1000)
        # 根据钉钉文档，正确的签名格式应为 timestamp\nsecret
        string_to_sign = f"{timestamp}\n{self.secret}"
        hmac_code = hmac.new(self.secret.encode(), string_to_sign.encode(), digestmod=hashlib.sha256).digest()
        sign = base64.b64encode(hmac_code).decode("utf-8")

        # 提取基础 URL（移除旧的时间戳和签名参数）
        base_url = self.webhook_url.split("&timestamp=")[0] if "&timestamp=" in self.webhook_url else self.webhook_url

        # 添加分隔符（? 或 &）
        separator = "?" if "?" not in base_url else "&"
        if separator == "&" and base_url.endswith("&"):
            separator = ""

        # 构建新的 URL
        self.webhook_url = f"{base_url}{separator}timestamp={timestamp}&sign={sign}"


class DingTalkCallback(WebhookCallback):
    """DingTalk notification callback with bilingual support."""

    def _init_bot(self) -> None:
        self.bot = DingTalkBot(self.webhook_url, self.secret)

    def send_msg(self, content: str) -> None:
        self.bot._check_sign()

        data = {
            "msgtype": "text",
            "text": {"content": content},
        }
        resp = requests.post(self.bot.webhook_url, json=data)
        resp.raise_for_status()
        result: Dict[str, Any] = resp.json()
        if result.get("errcode") and result["errcode"] != 0:
            print(f"❌ DingTalkBot sending failed: {result.get('errmsg')}")
            return
        print("✅ DingTalkBot sending successfully")

    def __str__(self):
        return "DingTalkBotCallback"


class WXWorkBot:
    """
    WxWork notification callback with bilingual support.
    docs: https://developer.work.weixin.qq.com/document/path/91770
    """

    def __init__(self, webhook_url: str):
        # WxWork does not need secret
        self.webhook_url = webhook_url


class WXWorkCallback(WebhookCallback):
    """WxWork notification callback with bilingual support."""

    def _init_bot(self) -> None:
        self.bot = WXWorkBot(self.webhook_url)

    def send_msg(self, content: str) -> None:
        data = {
            "msgtype": "text",
            "text": {"content": content},
        }
        resp = requests.post(self.bot.webhook_url, json=data)
        resp.raise_for_status()
        result: Dict[str, Any] = resp.json()
        if result.get("errcode") and result["errcode"] != 0:
            print(f"❌ WXWorkBot sending failed: {result.get('errmsg')}")
            return
        print("✅ WXWorkBot sending successfully")

    def __str__(self):
        return "WXWorkBotCallback"


class DiscordBot:
    """
    Discord notification callback with bilingual support.
    docs: https://discord.com/developers/docs/resources/webhook
    """

    def __init__(self, webhook_url: str):
        self.webhook_url = webhook_url


class DiscordCallback(WebhookCallback):
    """Discord notification callback with bilingual support."""

    def _init_bot(self) -> None:
        self.bot = DiscordBot(self.webhook_url)

    def send_msg(self, content: str) -> None:
        data = {
            "content": content,
        }
        resp = requests.post(self.bot.webhook_url, json=data)
        resp.raise_for_status()
        if resp.status_code not in [200, 204]:
            print(f"❌ DiscordBot sending failed: {resp.text}")
            return
        print("✅ DiscordBot sending successfully")

    def __str__(self):
        return "DiscordBotCallback"


class SlackBot:
    """
    Slack notification callback with bilingual support.
    docs: https://api.slack.com/messaging/webhooks
    """

    def __init__(self, webhook_url: str):
        self.webhook_url = webhook_url


class SlackCallback(WebhookCallback):
    """Slack notification callback with bilingual support."""

    def _init_bot(self) -> None:
        self.bot = SlackBot(self.webhook_url)

    def send_msg(self, content: str) -> None:
        data = {
            "text": content,
        }
        resp = requests.post(self.bot.webhook_url, json=data)
        resp.raise_for_status()
        if resp.status_code not in [200, 204]:
            print(f"❌ SlackBot sending failed: {resp.text}")
            return
        print("✅ SlackBot sending successfully")

    def __str__(self):
        return "SlackBotCallback"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/7/27 15:07
@File: setup.py
@IDE: pycharm
@Description:
    测试tutils/setup.py
"""
import pytest

import tutils.setup as SU


def test_mock_login_info():
    login_info = SU.mock_login_info()
    assert login_info.is_fail is False
    login_info = SU.mock_login_info(error_reason="Unauthorized")
    assert login_info.is_fail is True
    login_info = SU.mock_login_info(error_reason="Authorization Required")
    assert login_info.is_fail is True
    login_info = SU.mock_login_info(error_reason="Forbidden")
    assert login_info.is_fail is True
    login_info = SU.mock_login_info(error_reason="OK")
    assert login_info.is_fail is False


def test_use_setup_http():
    from swanlab.core_python import get_client
    from swanlab.data.store import get_run_store

    with SU.UseMockRunState() as run_state:
        assert run_state.client is not None
        assert get_client() is not None
        assert run_state.store is not None
        run_store = get_run_store()
        run_store.run_dir = "1"
        assert run_state.store.run_dir == "1"

    assert get_run_store().run_dir is None, "Run store should be reset after UseMockRunState context manager exits"
    with pytest.raises(ValueError):
        get_client()



"""
@author: cunyue
@file: cos.py
@time: 2025/6/16 13:30
@description: cos 对象，上传大文件到 Object Storage Service
"""

from concurrent.futures import ThreadPoolExecutor
from datetime import datetime, timedelta
from typing import List

import boto3
from botocore.config import Config as BotocoreConfig

from swanlab.data.modules import MediaBuffer
from swanlab.log import swanlog


class CosClient:
    REFRESH_TIME = 60 * 60 * 1.5  # 1.5小时

    def __init__(self, data):
        """
        初始化cos客户端
        """
        self.__expired_time = datetime.fromtimestamp(data["expiredTime"])
        self.__prefix = data["prefix"]
        self.__bucket = data["bucket"]
        credentials = data["credentials"]

        # 往期版本适配
        end_point = data.get("endPoint", None)
        path_style = 'path' if data.get('pathStyle', False) else 'virtual'
        endpoint_url = f"https://cos.{data['region']}.myqcloud.com" if end_point is None else end_point

        self.__client = boto3.client(
            's3',
            endpoint_url=endpoint_url,
            api_version='2006-03-01',
            aws_access_key_id=credentials['tmpSecretId'],
            aws_secret_access_key=credentials['tmpSecretKey'],
            aws_session_token=credentials['sessionToken'],
            config=BotocoreConfig(signature_version="s3", s3={'addressing_style': path_style}),
        )

    def upload(self, buffer: MediaBuffer):
        """
        上传文件，需要注意的是file_path应该为unix风格而不是windows风格
        开头不能有/
        :param buffer: 本地文件的二进制数据
        """
        key = "{}/{}".format(self.__prefix, buffer.file_name)
        try:
            swanlog.debug("Uploading file: {}".format(key))
            self.__client.put_object(
                Bucket=self.__bucket,
                Key=key,
                Body=buffer.getvalue(),
                # 一年
                CacheControl="max-age=31536000",
            )
        except Exception as e:
            swanlog.error("Upload error: {}".format(e))

    def upload_files(self, buffers: List[MediaBuffer]):
        """
        批量上传文件，keys和local_paths的长度应该相等
        :param buffers: 本地文件的二进制对象集合
        """
        # executor.submit可能会失败，因为线程数有限或者线程池已经关闭
        # 来自此issue: https://github.com/SwanHubX/SwanLab/issues/889，此时需要一个个发送
        failed_buffers = []
        with ThreadPoolExecutor(max_workers=10) as executor:
            futures = []
            for buffer in buffers:
                try:
                    futures.append(executor.submit(self.upload, buffer))
                except RuntimeError:
                    failed_buffers.append(buffer)
            for future in futures:
                future.result()
        # 重试失败的buffer
        if len(failed_buffers):
            swanlog.debug("Retrying failed buffers: {}".format(len(failed_buffers)))
            for buffer in failed_buffers:
                self.upload(buffer)

    @property
    def should_refresh(self):
        # cos传递的是北京时间，需要添加8小时
        # FIXME Use timezone-aware objects to represent datetimes in UTC; e.g. by calling .now(datetime.UTC)
        now = datetime.utcnow() + timedelta(hours=8)
        # 过期时间减去当前时间小于刷新时间，需要注意为负数的情况
        if self.__expired_time < now:
            return True
        return (self.__expired_time - now).seconds < self.REFRESH_TIME



"""
@author: cunyue
@file: qing_cloud.py
@time: 2024/11/18 15:14
@description: 青云(https://www.qingcloud.com/)元信息采集
"""

import os


BASE_KEYS = ['AICP_PLATFORM', 'AICP_TYPE', 'AICP_NAME', 'AICP_USER_NAME']
RESOURCES_KEYS = [
    'AICP_SPEC_COUNT',
    'AICP_SPEC_GPU',
    'AICP_SPEC_CPU',
    'AICP_SPEC_MEMORY',
    'AICP_SPEC_GPU_NAME',
    'AICP_SPEC_GPU_TYPE',
    'AICP_SPEC_GPU_MEMORY',
    'AICP_HOSTNAME',
    'AICP_HOST_MACHINE',
]


def get_qing_cloud_info():
    plat = os.getenv("AICP_PLATFORM")
    if not plat:
        return None
    return {**get_envs_by_keys(BASE_KEYS), "resources": get_envs_by_keys(RESOURCES_KEYS)}


def get_envs_by_keys(keys: list):
    """
    通过keys获取环境变量，最终返回一个dict，key为keys的值（小写），value为环境变量的值
    """
    return {key.lower(): os.getenv(key) for key in keys}



import numpy as np
import pytest

from swanlab.data.modules.object3d import PointCloud


class TestPointCloud:
    def test_init_validation(self, xyzrgb_points):
        """测试初始化验证"""
        # 正确的初始化
        pc = PointCloud(xyzrgb_points)
        assert pc.points.shape == (4, 6)

        # 错误的形状
        with pytest.raises(ValueError):
            PointCloud(np.zeros((4, 5)))

        # 错误的维度
        with pytest.raises(ValueError):
            PointCloud(np.zeros((4, 4, 3)))

    def test_from_xyz(self, xyz_points):
        """测试从XYZ创建"""
        pc = PointCloud.from_xyz(xyz_points)
        assert pc.points.shape == (4, 6)
        # 验证默认颜色是绿色
        assert np.all(pc.points[:, 3:] == [0, 255, 0])

    def test_from_xyzc(self, xyzc_points):
        """测试从XYZC创建"""
        pc = PointCloud.from_xyzc(xyzc_points)
        assert pc.points.shape == (4, 6)
        # 验证类别被正确映射到颜色
        assert pc.points.shape[1] == 6

    def test_from_xyzrgb(self, xyzrgb_points):
        """测试从XYZRGB创建"""
        pc = PointCloud.from_xyzrgb(xyzrgb_points)
        assert np.array_equal(pc.points, xyzrgb_points)

    def test_metadata(self, xyzrgb_points):
        """测试元数据处理"""
        pc = PointCloud(xyzrgb_points, caption="Test")
        assert pc.caption == "Test"
        assert pc.get_more() == {"caption": "Test"}

    def test_parse(self, xyzrgb_points):
        """测试解析方法"""
        pc = PointCloud(xyzrgb_points)
        filename, buffer = pc.parse()
        assert filename.endswith(".swanlab.pts.json")
        # 默认情况下step为None，所以就生成了None
        assert "stepNone" in filename
        assert buffer.getvalue()  # 确保buffer不为空

    def test_from_swanlab_pts_json_file(self, test_files_dir):
        """测试从文件加载"""
        file_path = test_files_dir / "points.swanlab.pts.json"
        pc = PointCloud.from_swanlab_pts_json_file(file_path)

        # 验证点云数据
        assert pc.points.shape == (4, 6)  # 4个点，每个点6个值

        # 验证具体的点
        expected_points = np.array(
            [
                [0, 0, 0, 255, 0, 0],  # 红色点
                [1, 0, 0, 0, 255, 0],  # 绿色点
                [0, 1, 0, 0, 0, 255],  # 蓝色点
                [0, 0, 1, 255, 255, 255],  # 白色点
            ],
            dtype=np.float32,
        )
        np.testing.assert_array_equal(pc.points, expected_points)

        # 测试错误情况
        with pytest.raises(FileNotFoundError):
            PointCloud.from_swanlab_pts_json_file(test_files_dir / "not_exist.json")

        # 测试带元数据的加载
        pc = PointCloud.from_swanlab_pts_json_file(file_path, step=1, caption="Test Points")
        assert pc.step == 1
        assert pc.caption == "Test Points"



from tutils import open_dev_mode
import swanlab

swanlab.login(open_dev_mode())

import os
from lightning import Trainer
from swanlab.integration.pytorch_lightning import SwanLabLogger
from tutils import open_dev_mode
from lightning_base import BoringModel, RandomDataset
from torch.utils.data import DataLoader


def main():
    print("User process PID:", os.getpid())

    # Set up data
    num_samples = 100000
    train = DataLoader(RandomDataset(32, num_samples), batch_size=32)
    val = DataLoader(RandomDataset(32, num_samples), batch_size=32)
    test = DataLoader(RandomDataset(32, num_samples), batch_size=32)

    model = BoringModel()

    swanlab_logger = SwanLabLogger(
        project="LightningTest",
        config={"num_samples": num_samples},
    )

    # Initialize a trainer
    trainer = Trainer(
        max_epochs=2,
        logger=swanlab_logger,
    )

    # Train the model
    trainer.fit(model, train, val)
    trainer.test(dataloaders=test)


if __name__ == "__main__":
    # swanlab.login(open_dev_mode())
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-01-01 16:08:15
@File: swanlab/data/__init__.py
@IDE: vscode
@Description:
    在此处完成回调注册、swanlog注册，并为外界提供api，提供运行时生成的配置
"""
from .run import (
    SwanLabRun as Run,
    SwanLabRunState as State,
    get_run,
    get_config,
    get_url,
    get_project_url,
    config,
)
from .sdk import (
    login,
    register_callbacks,
    init,
    log,
    finish,
    merge_settings,
)



"""
// @author: ComPleHN
// @file: image.vue
// @time: 2025/5/27 17:04
// @description: 本文件是对于echarts的 图片 的测试
"""
# ---------------------------------------------- Image - Image_base ----------------------------------------------
from pyecharts.components import Image
from pyecharts.options import ComponentTitleOpts


image = Image()

img_src = (
    "https://user-images.githubusercontent.com/19553554/"
    "71825144-2d568180-30d6-11ea-8ee0-63c849cfd934.png"
)
image.add(
    src=img_src,
    style_opts={"width": "200px", "height": "200px", "style": "margin-top: 20px"},
)
image.set_global_opts(
    title_opts=ComponentTitleOpts(title="Image-基本示例", subtitle="我是副标题支持换行哦")
)

c1 = image

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="image",
    public=True,
)

swanlab.log(
    {
        "Calendar - Calendar_heatmap": c1,
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/8 16:14
@File: test_project.py
@IDE: VSCode
@Description:
    测试开放API的项目相关接口
"""

import pytest

import tutils as T
from swanlab import OpenApi
from swanlab.api.types import ApiResponse, Project, Pagination


@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_list_projects():
    """
    测试列出一个 workspace 下的所有项目
    """
    api = OpenApi()
    resp = api.list_projects(detail=False)
    assert isinstance(resp, ApiResponse)
    if resp.code == 200:
        assert isinstance(resp.data, list)
        for item in resp.data:
            assert isinstance(item, Project)

@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_delete_project():
    """
    测试删除一个项目
    """
    api = OpenApi()
    project_name = "test_project"
    resp = api.delete_project(project=project_name)
    assert isinstance(resp, ApiResponse)
    assert resp.code in [204, 404]



import random
import sys

import nanoid

from swanlab import Settings
from swanlab.data.run.metadata.runtime import parse_git_url, get_python_info
from swanlab.swanlab_settings import set_settings


def test_parse_git_url():
    # ssh
    assert parse_git_url("git@github.com:swanhubx/swanlab.git") == "https://github.com/swanhubx/swanlab"
    assert parse_git_url("git@localhost:8000/swanhubx/swanlab.git") == "https://localhost:8000/swanhubx/swanlab"
    # https
    assert parse_git_url("https://github.com/swanhubx/swanlab.git") == "https://github.com/swanhubx/swanlab"
    assert parse_git_url("https://localhost:8000/swanhubx/swanlab.git") == "https://localhost:8000/swanhubx/swanlab"
    # no .git
    assert parse_git_url("git@github.com:swanhubx/swanlab") == "https://github.com/swanhubx/swanlab"
    assert parse_git_url("git@localhost:8000/swanhubx/swanlab") == "https://localhost:8000/swanhubx/swanlab"
    assert parse_git_url("https://github.com/swanhubx/swanlab") == "https://github.com/swanhubx/swanlab"
    assert parse_git_url("https://localhost:8000/swanhubx/swanlab") == "https://localhost:8000/swanhubx/swanlab"


def test_mask_no_api_key():
    """测试没有 api key 的情况下，是否替换"""
    _mock_args()
    # 不含有 api key，不替换
    cmd = get_python_info()["command"]
    assert ("****" not in cmd) is True


def test_mask_api_key_with_setting():
    """
    测试有 api key 的情况下，是否替换
    - 设置 security_mask 为 True，替换
    - 设置 security_mask 为 False，不替换
    """
    args = _mock_args()
    from tutils.setup import UseMockRunState

    with UseMockRunState() as run_state:
        client = run_state.client
        api_key = client.api_key

        # 隐藏隐私信息
        args[random.randint(0, len(args) - 1)] = api_key
        cmd = get_python_info()["command"]
        assert ("****" in cmd) is True

        # 设置为不主动替换隐私信息，即使有 api key 也不替换
        set_settings(Settings(security_mask=False))
        cmd = get_python_info()["command"]
        assert (api_key in cmd) is True
        assert ("****" not in cmd) is True


def _mock_args():
    """模拟生成一个随机长度的 args，并替换 sys.argv"""
    # 1 - 100 之间随机选取数组长度
    length = random.randint(1, 100)
    # 生成一个长度为 length 的随机字符串数组
    args = [nanoid.generate(size=random.randint(1, 20)) for _ in range(length)]
    sys.argv = args
    return args



#!/usr/bin/env python
r"""
@DATE: 2024/9/20 14:35
@File: namer.py
@IDE: pycharm
@Description:
    命名器、取色器
"""

import random
from typing import Optional, Tuple

prefix_list = [
    "swan-",  # 天鹅
    "rat-",  # 鼠
    "ox-",  # 牛
    "tiger-",  # 虎
    "rabbit-",  # 兔
    "dragon-",  # 龙
    "snake-",  # 蛇
    "horse-",  # 马
    "goat-",  # 羊
    "monkey-",  # 猴
    "rooster-",  # 鸡
    "dog-",  # 狗
    "pig-",  # 猪
    "cat-",  # 猫
    "elephant-",  # 象
    "penguin-",  # 企鹅
    "kangaroo-",  # 袋鼠
    "monkey-",  # 猴
    "panda-",  # 熊猫
    "tiger-",  # 虎
    "lion-",  # 狮
    "zebra-",  # 斑马
]


def generate_name(index: Optional[int] = None) -> str:
    """
    生成名称
    :param index: 生成名称的索引，约定为历史实验数，可以为None
    :return: 生成的名称
    """
    if index is None:
        prefix: str = random.choice(prefix_list)
        return prefix[:-1]
    else:
        prefix = prefix_list[index % len(prefix_list)]
        return prefix + str(index + 1)


light_colors = [
    "#528d59",  # 绿色
    "#587ad2",  # 蓝色
    "#c24d46",  # 红色
    "#9cbe5d",  # 青绿色
    "#6ebad3",  # 天蓝色
    "#dfb142",  # 橙色
    "#6d4ba4",  # 紫色
    "#8cc5b7",  # 淡青绿色
    "#892d58",  # 紫红色
    "#40877c",  # 深青绿色
    "#d0703c",  # 深橙色
    "#d47694",  # 粉红色
    "#e3b292",  # 淡橙色
    "#b15fbb",  # 浅紫红色
    "#905f4a",  # 棕色
    "#989fa3",  # 灰色
]

# 黑夜模式的颜色暂时和light_colors保持一致
dark_colors = [
    "#528d59",  # 绿色
    "#587ad2",  # 蓝色
    "#c24d46",  # 红色
    "#9cbe5d",  # 青绿色
    "#6ebad3",  # 天蓝色
    "#dfb142",  # 橙色
    "#6d4ba4",  # 紫色
    "#8cc5b7",  # 淡青绿色
    "#892d58",  # 紫红色
    "#40877c",  # 深青绿色
    "#d0703c",  # 深橙色
    "#d47694",  # 粉红色
    "#e3b292",  # 淡橙色
    "#b15fbb",  # 浅紫红色
    "#905f4a",  # 棕色
    "#989fa3",  # 灰色
]


def hex_to_rgb(hex_color: str) -> Tuple[int, int, int]:
    """将十六进制颜色转换为RGB格式

    Args:
        hex_color: 十六进制颜色字符串，支持以下格式：
            - "#528d59"
            - "528d59"
            - "#fff"  (简写形式)
            - "fff"

    Returns:
        Tuple[int, int, int]: RGB格式的颜色值，范围[0,255]

    Raises:
        ValueError: 当输入格式不正确时

    Examples:
        >>> hex_to_rgb("#528d59")
        (82, 141, 89)
        >>> hex_to_rgb("fff")  # 简写形式
        (255, 255, 255)
    """
    # 去除可能存在的#号和空白
    hex_color = hex_color.strip().lstrip("#").strip()

    # 验证输入
    if not all(c in "0123456789abcdefABCDEF" for c in hex_color):
        raise ValueError(f"Invalid hex color: {hex_color}")

    # 处理简写形式 (例如 #fff)
    if len(hex_color) == 3:
        hex_color = "".join(c * 2 for c in hex_color)
    elif len(hex_color) != 6:
        raise ValueError(f"Invalid hex color length: {len(hex_color)}, should be 3 or 6 characters")

    try:
        # 将16进制转换为RGB
        r = int(hex_color[:2], 16)
        g = int(hex_color[2:4], 16)
        b = int(hex_color[4:], 16)

        return (r, g, b)
    except ValueError as e:
        raise ValueError(f"Invalid hex color format: {hex_color}") from e


def generate_colors(index: Optional[int] = None) -> Tuple[str, str]:
    """
    生成颜色
    :param index: 生成颜色的索引，约定为历史实验数，可以为None
    :return: 生成的颜色，(白天颜色，夜晚颜色)
    """

    if index is None:
        choice_color = random.choice(light_colors)
        return choice_color, choice_color  # 随机返回一个颜色
    else:
        choice_color_light = light_colors[index % len(light_colors)]
        choice_color_dark = dark_colors[index % len(dark_colors)]
        return (
            choice_color_light,
            choice_color_dark,
        )  # 返回对应索引的颜色，如果超出范围则取模


def generate_run_id() -> str:
    """
    生成运行ID
    :return: 生成的运行ID，21个字符的小写字母和数字
    """
    return "".join(random.choices("abcdefghijklmnopqrstuvwxyz0123456789", k=21))



"""
@author: cunyue
@file: test_datastore.py
@time: 2025/6/5 18:33
@description: 测试 DataStore 类的功能，包括数据写入、读取、校验和计算等
"""

import os.path

from nanoid import generate

from swanlab.data.porter.datastore import DataStore
from tutils import TEMP_PATH

logs = [generate(size=l) for l in range(1, 100001, 1000)]


def test_write(filename=os.path.join(TEMP_PATH, "backup.swanlab")):
    """
    测试文件写入功能
    """
    ds = DataStore()
    ds.open_for_write(filename)
    for log in logs:
        ds.write(log)


def test_scan(filename=os.path.join(TEMP_PATH, "backup.swanlab")):
    """
    测试文件扫描功能
    """
    # 首先写入
    test_write(filename)
    # 然后扫描
    ds = DataStore()
    ds.open_for_scan(filename)
    for i in range(len(logs)):
        log = ds.scan()
        assert log == logs[i], "Error: Scanned log does not match written log"



"""
@author: cunyue
@file: cpu.py
@time: 2024/12/3 20:12
@description: CPU信息采集
"""

import multiprocessing
import platform
import subprocess

import psutil

from swanlab.data.run.metadata.hardware.type import HardwareFuncResult, HardwareCollector, HardwareInfoList
from swanlab.toolkit import is_macos
from .utils import CpuBaseCollector as C


def get_cpu_info() -> HardwareFuncResult:
    """获取 CPU 信息"""
    if is_macos():
        return None, None
    info = {"brand": None, "cores": None}

    # 获取 CPU 品牌, 根据不同操作系统调用不同的函数
    if platform.system() == "Windows":
        info["brand"] = get_cpu_brand_windows()
    elif platform.system() == "Linux":
        info["brand"] = get_cpu_brand_linux()
    else:
        # 其他情况，暂时不支持
        return None, None
    try:
        # 获取 CPU 核心数
        info["cores"] = multiprocessing.cpu_count()
    except Exception:  # noqa
        pass

    return info, CpuCollector()


def get_cpu_brand_windows():
    try:
        # 使用 WMIC 命令获取 CPU 品牌
        result = subprocess.run(["wmic", "cpu", "get", "name"], capture_output=True, text=True)
        cpu_brand = result.stdout.strip().split("\n")[-1].strip()
        return cpu_brand
    except Exception:  # noqa
        return None


def get_cpu_brand_linux():
    # lscpu 命令获取 CPU 品牌
    cpu_brand = None
    try:
        result = subprocess.run(["lscpu"], capture_output=True, text=True)
        for line in result.stdout.split("\n"):
            if "model name" in line.lower():
                cpu_brand = line.split(":")[1].strip()
                break
    except Exception:  # noqa
        pass
    if cpu_brand is None:
        try:
            # 使用 cat /proc/cpuinfo 获取CPU品牌
            with open("/proc/cpuinfo", "r") as f:
                for line in f:
                    if "model name" in line.lower():
                        cpu_brand = line.split(":")[1].strip()
                        break
        except Exception:  # noqa
            pass
    return cpu_brand


class CpuCollector(HardwareCollector, C):
    def __init__(self):
        super().__init__()
        self.current_process = psutil.Process()

    def collect(self) -> HardwareInfoList:
        return [
            self.get_cpu_usage(),
            # *self.get_per_cpu_usage(),
            self.get_cur_proc_thds_num(self.current_process),
        ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/1 17:30
@File: project.py
@IDE: pycharm
@Description:
    项目相关的开放API
"""

from swanlab.api.base import ApiBase, ApiHTTP
from swanlab.api.types import ApiResponse, Pagination, Project


class ProjectAPI(ApiBase):
    def __init__(self, http: ApiHTTP):
        super().__init__(http)

    @classmethod
    def parse(cls, body: dict, detail=True) -> Project:
        project_parser = {
            "cuid": body.get("cuid") or "",
            "name": body.get("name") or "",
            "description": body.get("description") or "",
            "visibility": body.get("visibility") or "",
            "createdAt": body.get("createdAt") or "",
            "updatedAt": body.get("updatedAt") or "",
            "group": {
                "type": (body.get("group") or {}).get("type") or "",
                "username": (body.get("group") or {}).get("username") or "",
                "name": (body.get("group") or {}).get("name") or "",
            },
        }
        if detail:
            project_parser["count"] = body.get("_count") or {}
        return Project.model_validate(project_parser)

    def delete_project(
        self, username: str, project: str
    ) -> ApiResponse[None]:
        """
        删除一个项目

        Args:
            username (str): 工作空间名
            project (str): 项目名
        """
        return self.http.delete(f"/project/{username}/{project}", params={})

    def list_projects(
        self, username: str, detail = True, page: int = 1, size: int = 10
    ) -> ApiResponse[Pagination[Project]]:
        """
        列出一个 workspace 下的所有项目

        Args:
            username (str): 工作空间名, 默认为用户个人空间
            detail (bool): 是否返回详细统计信息，默认为 True
            page (int): 页码，默认为 1
            size (int): 每页数量，默认为 10
        """
        resp = self.http.get(f"/project/{username}", params={"detail": detail, "page": page, "size": size})
        if resp.errmsg:
            return resp

        resp.data = Pagination[Project].model_validate({
            "total": resp.data.get("total", 0),
            "list": [ProjectAPI.parse(project, detail=detail) for project in resp.data.get("list", [])]
        })

        return resp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2023-12-19 19:00:37
@File: swanlab/utils/package.py
@IDE: vscode
@Description:
    用于管理swanlab的包管理器的模块，做一些封装
"""
import json
import netrc
import os
import re
from pathlib import Path
from typing import Optional

import requests

from .env import get_save_dir, SwanLabEnv
from .error import KeyFileError

package_path = os.path.join(os.path.dirname(__file__), "package.json")


# ---------------------------------- 版本号相关 ----------------------------------


def get_package_version() -> str:
    """获取swanlab的版本号
    :return: swanlab的版本号
    """
    # 读取package.json文件
    with open(package_path, "r") as f:
        return json.load(f)["version"]


def get_package_latest_version(timeout=0.5) -> Optional[str]:
    """
    获取swanlab的最新版本号
    :param timeout: 请求超时时间
    :return: 最新版本号
    """
    url = "https://pypi.org/pypi/swanlab/json"
    # noinspection PyBroadException
    try:
        response = requests.get(url, timeout=timeout)
        if response.status_code == 200:
            data = response.json()
            return data["info"]["version"]
        else:
            return None
    except Exception:
        return None


# ---------------------------------- 云端相关 ----------------------------------


class HostFormatter:
    def __init__(self, host: str = None, web_host: str = None):
        # 更新后的正则模式，允许匹配标准域名、IP地址和localhost
        self.pattern = re.compile(
            r'^(?:(https?)://)?'  # 可选协议 http 或 https
            r'('  # 主机部分（域名、IP、localhost）
            r'([a-zA-Z0-9.-]+\.[a-zA-Z]{2,63})'  # 标准域名
            r'|'  # 或
            r'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # IPv4地址
            r'|'  # 或
            r'(localhost)'  # 本地主机
            r')'
            r'(?::(\d{1,5}))?$'  # 可选端口号（1~5位数字）
        )
        self.host = host
        self.web_host = web_host

    def fmt(self, input_str: str) -> str:
        match = self.pattern.match(input_str.rstrip("/"))
        if match:
            protocol = match.group(1) or "https"  # 默认协议为 https
            host = match.group(2)
            port = match.group(6)

            # 构建标准化的 URL 输出
            result = f"{protocol}://{host}"
            if port:
                result += f":{port}"
            return result
        else:
            raise ValueError("Invalid host format")

    def __call__(self):
        """
        如果host或web_host不为空，格式化并设置环境变量
        :raises ValueError: host或web_host格式不正确
        """
        if self.host:
            try:
                os.environ[SwanLabEnv.API_HOST.value] = self.fmt(self.host) + "/api"
            except ValueError:
                raise ValueError("Invalid host: {}".format(self.host))
            self.web_host = self.host if self.web_host is None else self.web_host
        if self.web_host:
            try:
                os.environ[SwanLabEnv.WEB_HOST.value] = self.fmt(self.web_host)
            except ValueError:
                raise ValueError("Invalid web_host: {}".format(self.web_host))


def get_host_web() -> str:
    """获取swanlab网站网址
    :return: swanlab网站的网址
    """
    return os.getenv(SwanLabEnv.WEB_HOST.value, "https://swanlab.cn")


def get_host_api() -> str:
    """获取swanlab网站api网址
    :return: swanlab网站的api网址
    """
    return os.getenv(SwanLabEnv.API_HOST.value, "https://api.swanlab.cn/api")


def fmt_web_host(web_host: str = None) -> str:
    """
    如果web_host为None，则使用默认的web_host
    并且格式化web_host，去除结尾的/
    :param web_host: web_host
    :return: 格式化后的web_host
    """
    if web_host is None:
        web_host = get_host_web()
    return web_host.rstrip("/")


def get_setting_url(web_host: str = None) -> str:
    """获取用户设置的url
    与实验相关的url不同，这个url在http对象之前被使用，因此不绑定在http对象中
    :return: 用户设置的url
    """
    return fmt_web_host(web_host) + "/space/~/settings"


def get_login_url(web_host: str = None) -> str:
    """获取登录的url
    与实验相关的url不同，这个url在http对象之前被使用，因此不绑定在http对象中
    :return: 登录的url
    """
    return fmt_web_host(web_host) + "/login"


# ---------------------------------- 登录相关 ----------------------------------


def get_nrc_path() -> str:
    """
    获取netrc文件路径
    """
    return os.path.join(get_save_dir(), ".netrc")


def get_key():
    """使用标准netrc库解析token文件，获取token
    :raise KeyFileError: 文件不存在或者host不存在
    :return: token
    """
    env_key = os.getenv(SwanLabEnv.API_KEY.value)
    if env_key is not None:
        return env_key
    path, host = get_nrc_path(), get_host_api().rstrip("/api")
    if not os.path.exists(path):
        raise KeyFileError("The file does not exist")
    nrc = netrc.netrc(path)
    info = nrc.authenticators(host)

    # 向下兼容 https://github.com/SwanHubX/SwanLab/issues/792#issuecomment-2649685881
    if info is None:
        info = nrc.authenticators(host + "/api")
        if info is not None:
            nrc.hosts = {host: info}
            with open(path, "w") as f:
                f.write(repr(nrc))

    if info is None:
        raise KeyFileError(f"The host {host} does not exist")
    return info[2]


def save_key(username: str, password: str, host: str = None) -> bool:
    """
    保存key到对应的文件目录下，文件名称为.netrc（basename）
    此函数不考虑上层文件存在的清空，但是会在调用的get_save_dir()函数中进行检查
    :param username: 保存的用户名，默认为user，可选择存储为前端网页ip或者域名
    :param password: 保存的密码
    :param host: 保存的host
    :return: 是否保存，如果已经存在，则不保存
    """
    if host is None:
        host = get_host_api()
    host = host.rstrip()
    if host.endswith("/api"):
        host = host[:-4]
    path = get_nrc_path()
    if not os.path.exists(path):
        Path(path).touch()
    nrc = netrc.netrc(path)
    new_info = (username, "", password)
    # 避免重复的写
    info = nrc.authenticators(host)
    if info is None or (info[0], info[2]) != (new_info[0], new_info[2]):
        # 同时只允许存在一个host： https://github.com/SwanHubX/SwanLab/issues/797
        nrc.hosts = {host: new_info}
        with open(path, "w") as f:
            f.write(nrc.__repr__())
            try:
                f.flush()
                os.fsync(f.fileno())
            except OSError:
                # 如果操作系统不支持fsync，可能会抛出OSError，忽略此错误即可
                pass
        return True
    return False


class LoginCheckContext:
    """
    进入上下文时，会删除环境变量中的api key，退出上下文时会恢复原来的值
    """

    def __init__(self):
        self.__tmp_key = None
        """
        临时保存的key
        """
        self.is_login = False
        """
        标注是否已经登录
        """

    def __enter__(self):
        self.__tmp_key = os.environ.get(SwanLabEnv.API_KEY.value)
        if self.__tmp_key is not None:
            del os.environ[SwanLabEnv.API_KEY.value]
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        # 恢复原来的值
        if self.__tmp_key is not None:
            os.environ[SwanLabEnv.API_KEY.value] = self.__tmp_key
        if exc_type is KeyFileError:  # 未登录
            return True
        elif exc_type is not None:  # 其他错误
            return False
        self.is_login = True
        return True


def has_api_key() -> bool:
    """判断是否已经登录，与当前的host相关
    如果环境变量中有api key，则认为已经登录
    但不会检查key的有效性
    :return: 是否已经登录
    """
    with LoginCheckContext() as checker:
        _ = get_key()
    return checker.is_login



"""
------example.py------
from swanlab.converter import WandbConverter

wb_converter = WandbConverter()
wb_converter.run(wb_project="WANDB_PROJECT_NAME", wb_entity="WANDB_USERNAME")
"""

import swanlab
from swanlab.log import swanlog as swl
import time


class WandbConverter:
    def __init__(
        self,
        project: str = None,
        workspace: str = None,
        mode: str = "cloud",
        logdir: str = None,
        **kwargs,
    ):
        self.project = project
        self.workspace = workspace
        self.mode = mode
        self.logdir = logdir

    def parse_wandb_logs(self, wb_project: str, wb_entity: str, wb_run_id: str = None):
        try:
            import wandb
        except ImportError as e:
            raise TypeError(
                "Wandb Converter requires wandb. Install with 'pip install wandb'."
            )
            
        try:
            import pandas as pd
        except ImportError as e:
            raise TypeError(
                "Wandb Converter requires pandas when process wandb logs. Install with 'pip install pandas'."
            )

        client = wandb.Api()

        if wb_run_id is None:
            # process all runs
            runs = client.runs(wb_entity + "/" + wb_project)
        else:
            # get the run by run_id
            run = client.run(f"{wb_entity}/{wb_project}/{wb_run_id}")
            runs = (run,)

        for iter, wb_run in enumerate(runs):
            swl.info(f"Conversion progress: {iter+1}/{len(runs)}")

            if swanlab.get_run() is None:
                swanlab_run = swanlab.init(
                    project=wb_project if self.project is None else self.project,
                    workspace=self.workspace,
                    experiment_name=wb_run.name,
                    description=wb_run.notes,
                    mode=self.mode,
                    logdir=self.logdir,
                )
            else:
                swanlab_run = swanlab.get_run()
                
            try:
                wb_run_metadata = {"wandb_metadata": wb_run.metadata}
            except:
                wb_run_metadata = {}

            wb_config = {
                "wandb_run_id": wb_run.id,
                "wandb_run_name": wb_run.name,
                "Created Time": wb_run.created_at,
                "wandb_user": wb_run.user,
                "wandb_tags": wb_run.tags,
                "wandb_url": wb_run.url,
            }
            wb_config.update(wb_run_metadata)

            swanlab_run.config.update(wb_config)
            swanlab_run.config.update(wb_run.config)

            # Get the first history record to extract available keys
            history = wb_run.history(stream="default")
            if len(history) > 0:
                # 检查 history 是否为 DataFrame 类型
                if isinstance(history, pd.DataFrame):
                    # 如果是 DataFrame，直接获取列名
                    keys = [key for key in history.columns if not key.startswith("_")]
                else:
                    # 原来的逻辑，假设是字典列表
                    keys = [key for key in history[0].keys() if not key.startswith("_")]
            else:
                keys = []

            data_pool = {}
            
            # 记录标量指标
            for record in wb_run.scan_history():
                step = record.get("_step")
                for key in keys:
                    value = record.get(key)
                    # 如果value是None或者是dict类型，则跳过
                    if value is None or isinstance(value, dict) or not isinstance(value, (float, int)):
                        # 如果是多媒体数据，如图像，value是这个格式
                        # image {'format': 'png', 'path': 'media/images/image_13_3bbb7517118b6af0307c.png', 'sha256': '3bbb7517118b6af0307cbe1b26f6d94b68797874112de716df4b5b50e01ddc24', 'size': 30168, 'height': 100, 'width': 100, '_type': 'image-file'}
                        continue
                    
                    # 使用dict存储数据
                    if key not in data_pool:
                        data_pool[key] = []
                    data_pool[key].append({"value": value, "step": step})
                    
            # 分key记录数据，防止阻塞
            index = 0
            for key, data_list in data_pool.items():
                for data in data_list:
                    swanlab_run.log({key: data["value"]}, step=int(data["step"]))
                # TODO: 等未来上传方案优化后解除延时
                if index % 5 == 0:
                    time.sleep(1)
                print(f"Index {index} / {len(data_pool)}: Metric: {key} Data Number: {len(data_list)}")
                index += 1

            # 结束此轮实验
            swanlab_run.finish()

    def run(self, wb_project: str, wb_entity: str, wb_run_id: str = None):
        swl.info("Start converting Wandb Runs to SwanLab...")
        self.parse_wandb_logs(
            wb_project=wb_project,
            wb_entity=wb_entity,
            wb_run_id=wb_run_id,
        )
        swl.info("Finished converting Wandb Runs to SwanLab.")



"""
@author: cunyue
@file: __init__.py
@time: 2024/12/3 20:17
@description: 合作信息采集
"""

from typing import TypedDict, Optional

from swanlab.core_python import get_client
from swanlab.env import get_mode
from swanlab.package import get_package_version
from swanlab.toolkit import get_swanlog_dir
from .qing_cloud import get_qing_cloud_info


class SwanLabInfo(TypedDict):
    version: str
    mode: str
    swanlog_dir: str
    exp_url: str


class CoopInfo(TypedDict):
    swanlab: SwanLabInfo
    qing_cloud: dict


def get_cooperation_info(swanlab: bool = False) -> Optional[CoopInfo]:
    """获取第三方合作信息"""
    qc = get_qing_cloud_info()
    coop = {}
    if swanlab:
        coop.update({"swanlab": get_swanlab_info()})
    if qc:
        coop.update({"qing_cloud": qc})
    return coop if len(coop) > 0 else None


def get_swanlab_info() -> SwanLabInfo:
    data: SwanLabInfo = {
        "version": get_package_version(),
        "mode": get_mode(),
        "swanlog_dir": get_swanlog_dir(),
    }
    try:
        client = get_client()
        data["exp_url"] = client.web_exp_url
    except ValueError:
        pass
    return data



import pytest

from swanlab.data.run.metadata.hardware.gpu.moorethreads import MTTCollector, map_moorethreads_gpu

try:
    driver, gpu_map = map_moorethreads_gpu()
    max_mem_value = 0
    for gpu_id in gpu_map:
        gpu_mem = int(gpu_map[gpu_id].get("memory", 0))
        max_mem_value = max(max_mem_value, gpu_mem)
    max_mem_value = max_mem_value * 1024
except Exception as e:
    driver = None
    gpu_map = None
    max_mem_value = None


@pytest.mark.skipif(driver is None, reason="MooreThreads GPU not found")
def test_moorethreads_gpu_info():
    collector = MTTCollector(gpu_map, max_mem_value)
    collector()
    assert collector.collect_num == 1



"""
@author: cunyue
@file: model.py
@time: 2025/6/16 14:12
@description: 定义上传模型
"""

import json
from datetime import datetime
from enum import Enum
from typing import List, Optional, TypedDict, Literal

from swanlab.data.modules import MediaBuffer
from swanlab.toolkit import ColumnClass, ColumnConfig


class ColumnModel:
    """
    列信息上传模型
    """

    def __init__(
        self,
        key,
        name: Optional[str],
        cls: ColumnClass,
        typ: str,
        config: Optional[ColumnConfig],
        section_name: Optional[str],
        section_type: Optional[str],
        error: dict = None,
    ):
        """
        key: 键
        name: 键的名称
        cls: 键的类别
        typ: 键的类型
        config: 键的配置
        section_name: 键所在的section的名称
        section_type: 键所在的section的类型
        error: 错误信息
        """
        self.key = key
        self.name = name
        self.cls = cls
        self.typ = typ
        self.config = config
        self.section_name = section_name
        self.section_type = section_type
        self.error = error

    def to_dict(self):
        """
        序列化为Dict,传递给后端
        """
        d = {
            "class": self.cls,
            "type": self.typ,
            "key": self.key,
            "name": self.name,
            "error": self.error,
            "sectionName": self.section_name,
            "sectionType": self.section_type,
        }
        if self.name is None:
            d.pop("name")
        if self.error is None:
            d.pop("error")
        if self.section_name is None:
            d.pop("sectionName")
        if self.section_type is None:
            d.pop("sectionType")
        if self.config is None:
            return d
        # 将额外的图表配置信息加入
        if self.config.y_range is not None:
            d["yRange"] = self.config.y_range
        if self.config.chart_name is not None:
            d["chartName"] = self.config.chart_name
        if self.config.chart_index is not None:
            d["chartIndex"] = self.config.chart_index
        if self.config.metric_name is not None:
            d["metricName"] = self.config.metric_name
        if self.config.metric_color is not None:
            d["metricColors"] = self.config.metric_color
        return d


class MetricType(Enum):
    """
    指标类型枚举
    """

    SCALAR = "scalar"
    """
    标量指标
    """
    MEDIA = "media"
    """
    媒体指标
    """
    LOG = "log"
    """
    日志数据
    """


class MediaModel:
    """
    媒体指标信息上传模型
    """

    type = MetricType.MEDIA

    def __init__(
        self,
        metric: dict,
        key: str,
        key_encoded: str,
        step: int,
        epoch: int,
        buffers: List[MediaBuffer] = None,
    ):

        # -------------------------- 🤡这里是一点小小的💩 --------------------------
        # 要求上传时的文件路径必须带key_encoded前缀
        if buffers is not None:
            metric = json.loads(json.dumps(metric))
            for i, d in enumerate(metric["data"]):
                metric["data"][i] = "{}/{}".format(key_encoded, d)
        # ------------------------------------------------------------------------

        self.metric = metric
        self.step = step
        self.epoch = epoch
        self.key = key
        """
        真实的指标名称
        """
        self.key_encoded = key_encoded
        """
        编码后路径安全的指标名称
        """
        self.buffers = buffers
        """
        原始数据，可能为None
        """

    def to_dict(self):
        """
        序列化
        """
        return {
            **self.metric,
            "key": self.key,
            "index": self.step,
            "epoch": self.epoch,
        }


class ScalarModel:
    """
    标量指标信息上传模型
    """

    type = MetricType.SCALAR

    def __init__(self, metric: dict, key: str, step: int, epoch: int):
        self.metric = metric
        self.key = key
        self.step = step
        self.epoch = epoch

    def to_dict(self):
        """
        序列化
        """
        return {
            **self.metric,
            "key": self.key,
            "index": self.step,
            "epoch": self.epoch,
        }


class FileModel:
    """
    运行时文件信息上传模型
    """

    def __init__(
        self,
        requirements: str = None,
        metadata: dict = None,
        config: dict = None,
        conda: str = None,
    ):
        self.requirements = requirements
        self.metadata = metadata
        self.config = config
        self.conda = conda
        # 主要用于去重，保留最新的文件
        self.create_time = datetime.now()

    @classmethod
    def create(cls, file_models: List["FileModel"]) -> "FileModel":
        """
        比较若干个FileModel，获取最新的FileModel，并且保证其内部属性不为None
        """
        # 按照时间排序，倒叙排列，这意味着最新的排在第一个
        file_models = sorted(file_models, key=lambda x: x.create_time, reverse=True)
        lr, lm, lc, lo = None, None, None, None
        for file_model in file_models:
            lr = file_model.requirements if lr is None else lr
            lm = file_model.metadata if lm is None else lm
            lc = file_model.config if lc is None else lc
            lo = file_model.conda if lo is None else lo
            if lr is not None and lm is not None and lc is not None and lo is not None:
                break
        return FileModel(lr, lm, lc, lo)

    def to_dict(self):
        """
        序列化，会删除为None的字段
        """
        d = {
            "requirements": self.requirements,
            "metadata": self.metadata,
            "config": self.config,
            "conda": self.conda,
        }
        return {k: v for k, v in d.items() if v is not None}

    @property
    def empty(self):
        """
        是否为空
        """
        return all(getattr(self, attr) is None for attr in ('requirements', 'metadata', 'config', 'conda'))


class LogModel(TypedDict):
    """
    日志信息上传模型
    """

    level: Literal['INFO', 'WARN', 'ERROR']
    """
    日志级别
    """
    contents: List[dict]
    """
    当前日志级别的日志内容
    """


__all__ = [
    "LogModel",
    "ColumnModel",
    "ScalarModel",
    "MediaModel",
    "FileModel",
]
